/** Copyright 2012
*
* Author: Ismael PONCE GORDILLO
*
* Licensed under the EUPL, Version 1.1 or – as soon they will be
* approved by the European Commission – subsequent versions of the
* EUPL (the "Licence"); you may not use this work except in
* compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* https://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the Licence is distributed on an "AS
* IS" BASIS, * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
* express or implied.
* See the Licence for the specific language governing permissions
* and limitations under the Licence.
*/
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Calendar;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
//
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.FileDialog;

public class BBDD {
	public static final String dirBBDD=PCuaderno.MiPCuaderno.titulo+"_bbdd"+File.separatorChar;
	public static final String nombreBBDD="bbdd.hsqldb";
	public static final String archivo=BBDD.dirBBDD+BBDD.nombreBBDD;
	public static final String archivoLCK=BBDD.archivo+".lck";
	public static final String archivoLOG=BBDD.archivo+".log";
	public static final String archivoSCRIPT=BBDD.archivo+".script";
	public static final String archivoPROPERTIES=BBDD.archivo+".properties";
	public static final String archivoDUMP=BBDD.archivo+".dump";
	private static String cadenaConexion=null;
	private static String cadenaConexionCierre=";shutdown=true";
	public static final int idTipoEvaluacion=1;
	public static final int idTipoClaustro=2;
	public static final int idTipoConsejoEscolar=3;
	public static final int idTipoCitaConTutores=4;
	public static final int idTipoETCP=5;
	public static final int idTipoOtroTipo=6;
	public static final String expTipoTodos="Todos";
	public static final int idTipoTodos=0;
	public static final String verdadero="TRUE";
	public static final String ordinario="Ordinario";
	public static final String extraordinario="Extraordinario";
	public static final String sqlDemoradoINFO="<<INFO>>";
	public static final String expSinGrupoEvaluacion="Sin grupo";
	public static final String expNuevoGrupoEvaluacion="GrupoNuevo";
	public static final String expNoMoverGrupo="";
	public static final String expDiaCompleto="Dia Completo";
	public static final String idDiaCompleto="#DiaCompleto";
	public static final String expTodos="Todos";
	public static final String expTodas="Todas";
	public static final String idTodos="#Todos";
	public static final String conjuntoCaracteresEstudiantes="ISO-8859-1";//ISO-8859-1,UTF-8,windows-1250
	public static final String conjuntoCaracteresHSQLDB="Unicode";
	public static final int bufferArchivo=2048;
	public static boolean acceso(String cadenaAcceso){
		BBDD.cadenaConexion="jdbc:hsqldb:file:"+BBDD.archivo+";crypt_key="+cadenaAcceso+";crypt_type=AES";
		//BBDD.cadenaConexion="jdbc:hsqldb:file:"+BBDD.archivo+";";
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+cadenaAcceso+") longitud "+cadenaAcceso.length());
		return compruebaConexion();
	}
	public static boolean compruebaConexion(){
		boolean resultado=true;
		Connection tConexion = null;
		try {tConexion = DriverManager.getConnection(BBDD.cadenaConexion);tConexion.close();
		} catch (SQLException e) {
			System.out.println("BBDD.compruebaConexion - No pudo conectar con la base de datos.");
			System.out.println(e.toString());
			resultado=false;
			}
		return resultado;
	}

	public static void borrarParcial(String idParcial){
		int ejecucion=-1;
		if (idParcial!=null) {
			if (!idParcial.equals("")) {
				String sql="DELETE FROM PARCIALES WHERE PARCIALES.ID="+idParcial+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idParcial+") borrados "+ejecucion);
	}
	public static String insertarParcial(String idMateria,Calendar inicio, Calendar fin){
		String id=null;
		if (idMateria!=null && inicio!=null && fin!=null) if (!idMateria.isEmpty()){
			String sql="INSERT INTO PARCIALES (MATERIA,INICIO,FIN)"+
					" VALUES ("+idMateria+",'"+Fecha.bbddSoloDia(inicio)+"','"+Fecha.bbddSoloDia(fin)+"');";
			id = BBDD.insertarIdSQL("PARCIALES",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+","+Fecha.formaFechaDiaHora(inicio)+","+Fecha.formaFechaDiaHora(fin)+") id del insertado "+id);
		return id;
	}
	public static void modificarParcial(String idParcial, Calendar inicio, Calendar fin){
		int ejecucion=-1;
		if (idParcial!=null && inicio!=null && fin!=null) {
			if (!idParcial.isEmpty()) {
				String sql="UPDATE PARCIALES SET PARCIALES.INICIO='"+Fecha.bbddSoloDia(inicio)+
						"',PARCIALES.FIN='"+Fecha.bbddSoloDia(fin)+"'"+
				" WHERE PARCIALES.ID="+idParcial+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idParcial+","+Fecha.formaFechaDiaHora(inicio)+") modificados "+ejecucion);
	}
	public static Vector<Vector<String>> consultarParcial(String idMateria,String idParcial){
		Vector<Vector<String>> tabla=null;
		if (idMateria!=null) if (!idMateria.isEmpty()) {
			String sql="SELECT ID,MATERIA,DAYOFMONTH(INICIO),MONTH(INICIO),YEAR(INICIO)" +
					",DAYOFMONTH(FIN),MONTH(FIN),YEAR(FIN) FROM PARCIALES"+
					" WHERE PARCIALES.MATERIA="+idMateria+
					((idParcial!=null)?" AND PARCIALES.ID="+idParcial:"")+
					" ORDER BY INICIO ASC;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+","+idParcial+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEnsenanza(String idEnsenanza){
		String sql="SELECT ENSENANZAS.ID,ENSENANZAS.REFERENCIA,ENSENANZAS.DESCRIPCION FROM ENSENANZAS"+
				((idEnsenanza!=null)?" WHERE ENSENANZAS.ID="+idEnsenanza:"")+
				" ORDER BY REFERENCIA;";
		Vector<Vector<String>> tabla = BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEnsenanza+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static String insertarEnsenanza(String referencia, String ensenanza){
		String id=null;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+referencia+","+ensenanza+") "+id);
		if (referencia!=null && ensenanza!=null) 
			if (!referencia.isEmpty()) {
				String sql="INSERT INTO ENSENANZAS (REFERENCIA,DESCRIPCION)"+
			" VALUES ('"+referencia+"','"+ensenanza+"');";
			id = BBDD.insertarIdSQL("ENSENANZAS",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+referencia+","+ensenanza+") id del insertado = "+id);
		return id;
	}
	public static void borrarEnsenanza(String idEnsenanza){
		int ejecucion=-1;
		if (idEnsenanza!=null) {
			if (!idEnsenanza.equals("")) {
				Vector<Vector<String>> titulos = BBDD.consultarTitulo(idEnsenanza,null);
				if (titulos!=null) if (titulos.size()==0) {
					String sql="DELETE FROM ENSENANZAS WHERE ENSENANZAS.ID="+idEnsenanza+";";
					ejecucion=BBDD.ejecutaSQL(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEnsenanza+") borrados "+ejecucion);
	}
	public static Vector<Vector<String>> consultarTitulo(String idEnsenanza,String idTitulo){
		String sql="SELECT TITULOS.ID,TITULOS.ENSENANZA,TITULOS.REFERENCIA,TITULOS.DESCRIPCION FROM TITULOS"+
				((idEnsenanza!=null || idTitulo!=null)?" WHERE":"")+
				((idEnsenanza!=null)?" TITULOS.ENSENANZA="+idEnsenanza:"")+
				((idEnsenanza!=null && idTitulo!=null)?" AND":"")+
				((idTitulo!=null)?" TITULOS.ID="+idTitulo:"")+
				" ORDER BY REFERENCIA;";
		Vector<Vector<String>> tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEnsenanza+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static String insertarTitulo(String idEnsenanza,String referencia,String descripcion){
		String id=null;
		if (idEnsenanza!=null && referencia!=null && descripcion!=null) 
			if (!idEnsenanza.isEmpty() && !referencia.isEmpty()) {
				String sql="INSERT INTO TITULOS (ENSENANZA,REFERENCIA,DESCRIPCION)"+
			" VALUES ('"+idEnsenanza+"','"+referencia+"','"+descripcion+"');";
			id = BBDD.insertarIdSQL("TITULOS",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEnsenanza+","+referencia+","+descripcion+") id del insertado = "+id);
		return id;
	}
	public static void borrarTitulo(String idTitulo){
		int ejecucion=-1;
		if (idTitulo!=null) {
			if (!idTitulo.equals("")) {
				Vector<Vector<String>> titulos = BBDD.consultarNivel(idTitulo,null);
				if (titulos!=null) if (titulos.size()==0) {
					String sql="DELETE FROM TITULOS WHERE TITULOS.ID="+idTitulo+";";
					ejecucion=BBDD.ejecutaSQL(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTitulo+") borrados "+ejecucion);
	}
	public static Vector<Vector<String>> consultarNivel(String idTitulo,String idNivel){
		String sql="SELECT NIVELES.ID,NIVELES.TITULO,NIVELES.REFERENCIA FROM NIVELES"+
				((idTitulo!=null || idNivel!=null)?" WHERE":"")+
				((idTitulo!=null)?" NIVELES.TITULO="+idTitulo:"")+
				((idTitulo!=null && idNivel!=null)?" AND":"")+
				((idNivel!=null)?" NIVELES.ID="+idNivel:"")+
				" ORDER BY NIVELES.REFERENCIA;";
		Vector<Vector<String>> tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTitulo+","+idNivel+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static String insertarNivel(String idTitulo,String referencia){
		String id=null;
		if (idTitulo!=null && referencia!=null) 
			if (!idTitulo.isEmpty() && !referencia.isEmpty()) {
				String sql="INSERT INTO NIVELES (TITULO,REFERENCIA)"+
			" VALUES ("+idTitulo+",'"+referencia+"');";
			id = BBDD.insertarIdSQL("NIVELES",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTitulo+","+referencia+") id del insertado = "+id);
		return id;
	}
	public static void borrarNivel(String idNivel){
		int ejecucion=-1;
		if (idNivel!=null) {
			if (!idNivel.equals("")) {
				Vector<Vector<String>> titulos = BBDD.consultarClase(idNivel,null);
				if (titulos!=null) if (titulos.size()==0) {
					String sql="DELETE FROM NIVELES WHERE NIVELES.ID="+idNivel+";";
					ejecucion=BBDD.ejecutaSQL(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+") borrados "+ejecucion);
	}
	public static String insertarClase(String idNivel,String referencia){
		String id=null;
		int ejecucion=0;
		String tutor=null;
		Vector<Vector<String>> materias=null;
		Vector<Vector<String>> profesores=BBDD.consultarProfesor(null);
		if (profesores!=null) if (profesores.size()>0) tutor=profesores.get(0).get(0);
		if (idNivel!=null && referencia!=null && tutor!=null) 
			if (!idNivel.isEmpty() && !referencia.isEmpty()) {
				String sql="INSERT INTO CLASES (NIVEL,REFERENCIA,TUTOR)" +
						" VALUES ("+idNivel+",'"+referencia+"',"+tutor+");";
				id = BBDD.insertarIdSQL("CLASES",sql);
				sql="";
				materias = BBDD.consultarMateria(idNivel, null);
				if (materias!=null) if (materias.size()>0) {
					for (int i=0;i<materias.size();i++) 
						sql+="INSERT INTO MATERIA_CLASE (MATERIA,CLASE) VALUES ("+materias.get(i).get(0)+","+id+");";
				}
				ejecucion=BBDD.ejecutaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+","+referencia+") tutor ("+tutor+") materias ("+((materias!=null)?materias.size():"null")+") id del insertado = "+id+" insertados "+ejecucion);
		return id;
	}
	public static void borrarClase(String idClase){
		int ejecucion=-1;
		Vector<Vector<String>> estudiantes=null;
		Vector<Vector<String>> uhs=null;
		Vector<Vector<String>> evaluaciones=null;
		if (idClase!=null) {
			if (!idClase.equals("")) {
				estudiantes = BBDD.consultarEstudiantesClase(idClase,null);
				if (estudiantes==null || estudiantes.size()==0) {
					evaluaciones = BBDD.consultaSQL("SELECT EVALUACIONES.ID FROM EVALUACIONES WHERE EVALUACIONES.CLASE="+idClase+";");
					String sql="";
					if (evaluaciones!=null) if (evaluaciones.size()>0) {
						for (int i=0;i<evaluaciones.size();i++) sql+="DELETE FROM EVALUACIONES WHERE EVALUACIONES.ID="+evaluaciones.get(i).get(0)+";"+
								" DELETE FROM EVENTOS WHERE EVENTOS.ID="+evaluaciones.get(i).get(0)+";";
					}
					uhs = BBDD.consultarUHMateriaClase(null, idClase);
					if (uhs!=null) if (uhs.size()>0) {
						for (int i=0;i<uhs.size();i++) sql+="DELETE FROM UH_SEMANA WHERE UH_SEMANA.ID="+uhs.get(i).get(0)+";";
					}
					sql+="DELETE FROM PROGRAMACION_TEMPORAL WHERE PROGRAMACION_TEMPORAL.CLASE="+idClase+";"+
							" DELETE FROM DIASNOLECTIVOS WHERE DIASNOLECTIVOS.CLASE="+idClase+";"+
							" DELETE FROM MATERIA_CLASE WHERE MATERIA_CLASE.CLASE="+idClase+";"+
							" DELETE FROM CLASES WHERE CLASES.ID="+idClase+";";
					ejecucion=BBDD.ejecutaSQL(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") estudiantes ("+((estudiantes!=null)?estudiantes.size():"null")+") evaluaciones ("+((evaluaciones!=null)?evaluaciones.size():"null")+") horarios ("+((uhs!=null)?uhs.size():"null")+") total borrados "+ejecucion);
	}
	public static void borrarMateria(String idMateria){
		int ejecucion=-1;
		Vector<Vector<String>> estudiantes=null;
		Vector<Vector<String>> tiposEvaluable=null;
		Vector<Vector<String>> uhs=null;
		Vector<Vector<String>> nodos=null;
		if (idMateria!=null) {
			if (!idMateria.equals("")) {
				estudiantes = BBDD.consultarEstudiantesMateria(idMateria,null);
				tiposEvaluable = BBDD.consultarTiposEvaluablesParaMateria(idMateria);
				if ((estudiantes==null||estudiantes.size()==0) && (tiposEvaluable==null ||tiposEvaluable.size()==0)) {
					String sql="DELETE FROM PARCIALES WHERE PARCIALES.MATERIA="+idMateria+";";
					nodos = BBDD.consultaSQL("SELECT ID FROM PROGRAMACION_NODO WHERE PROGRAMACION_NODO.MATERIA="+idMateria+";");
					for (int i=0;i<nodos.size();i++) {
						sql+="DELETE FROM PROGRAMACION_TEMPORAL WHERE PROGRAMACION_TEMPORAL.NODO="+nodos.get(i).get(0)+";";
					}
					sql+="DELETE FROM PROGRAMACION_NODO WHERE PROGRAMACION_NODO.MATERIA="+idMateria+";";
					uhs = BBDD.consultarUHMateriaClase(idMateria,null);
					if (uhs.size()>0) {
						for (int i=0;i<uhs.size();i++) sql+="DELETE FROM UH_SEMANA WHERE UH_SEMANA.ID="+uhs.get(i).get(0)+";";
					}
					sql+="DELETE FROM MATERIA_CLASE WHERE MATERIA_CLASE.MATERIA="+idMateria+";" +
							"DELETE FROM MATERIAS WHERE MATERIAS.ID="+idMateria+";";
					ejecucion=BBDD.ejecutaSQL(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+")" +" estudiantes ("+((estudiantes!=null)?estudiantes.size():"null")+")" +" tiposEvaluables ("+((tiposEvaluable!=null)?tiposEvaluable.size():"null")+")" +" nodos ("+((nodos!=null)?nodos.size():"null")+")" +" uhs ("+((uhs!=null)?uhs.size():"null")+") total borrados "+ejecucion);
	}
	public static Vector<Vector<String>> consultarMateria(String idNivel,String idMateria){
		Vector<Vector<String>> tabla = new Vector<Vector<String>>();
		String sql="SELECT MATERIAS.ID,MATERIAS.REFERENCIA,MATERIAS.DESCRIPCION,MATERIAS.NIVEL FROM MATERIAS"+
				((idNivel!=null || idMateria!=null)?" WHERE":"")+
				((idNivel!=null)?" MATERIAS.NIVEL="+idNivel:"")+
				((idNivel!=null && idMateria!=null)?" AND":"")+
				((idMateria!=null)?" MATERIAS.ID="+idMateria:"")+
				" ORDER BY MATERIAS.REFERENCIA;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+","+idMateria+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static void modificarTipoDeEvaluable(String idEvaluable, String idTipo){
		int ejecucion=-1;
		if (idEvaluable!=null && idTipo!=null) {
			if (!idEvaluable.equals("") && !idTipo.equals("")) {
				String sql="UPDATE EVALUABLES SET EVALUABLES.TIPO="+idTipo+
				" WHERE EVALUABLES.ID="+idEvaluable+";";
				//System.out.println("sql "+sql);
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+","+idTipo+") modificados "+ejecucion);
	}
	public static String insertarProgramacionNodo(String enunciado, String idPadre, Integer posicion, String idMateria){
		String id=null;
		if (idPadre!=null && posicion!=null && idMateria!=null) 
			if (!idPadre.isEmpty() && posicion>=0 && !idMateria.isEmpty()) {
				String sql="INSERT INTO PROGRAMACION_NODO (PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO)"+
			" VALUES ("+idPadre+","+posicion+","+idMateria+",'"+enunciado+"','');";
			id = BBDD.insertarIdSQL("PROGRAMACION_NODO",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+enunciado+","+idPadre+","+posicion+","+idMateria+") id del insertado = "+id);
		return id;
	}
	public static String insertarProgramacionNodoHermano(String idHermano){
		String id=null;
		Vector<Vector<String>> hermanosPosteriores=null;
		if (idHermano!=null) 
			if (!idHermano.isEmpty()) {
				Vector<Vector<String>> nodo = BBDD.consultarProgramacionNodo(idHermano);
				String idPadre = nodo.get(0).get(1);
				Integer posNodo = Integer.valueOf(nodo.get(0).get(2));
				//System.out.println("BBDD insertarProgramcionNodoHermano (padre "+idPadre+", posNodo"+posNodo+")");
				hermanosPosteriores=null;
				String sql="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE"+((idPadre==null)?" IS NULL AND MATERIA="+nodo.get(0).get(3):"="+idPadre)+
						" AND POSICION>"+posNodo+
						" ORDER BY POSICION ASC;";
				hermanosPosteriores=BBDD.consultaSQL(sql);
				//System.out.println("BBDD insertarProgramcionNodoHermano (hermanos "+hermanosPosteriores.size()+")");
				sql="";
				if (hermanosPosteriores!=null) if (hermanosPosteriores.size()>0) {
					for (int i=0;i<hermanosPosteriores.size();i++){
						sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(i+posNodo+2)+
						" WHERE PROGRAMACION_NODO.ID="+hermanosPosteriores.get(i).get(0)+";";
					}
				}
				sql+="INSERT INTO PROGRAMACION_NODO (PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO)" +
						" VALUES ("+((idPadre==null)?"NULL":idPadre)+","+(posNodo+1)+","+nodo.get(0).get(3)+",'NUEVO','');";
				id = BBDD.insertarIdSQL("PROGRAMACION_NODO",sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idHermano+") hermanos posteriores ("+((hermanosPosteriores!=null)?hermanosPosteriores.size():"null")+")id del insertado = "+id);
		return id;
	}
	public static void insertarProgramacionDuracionNodo(String idNodo, String idClase, Integer duracion){
		int ejecucion=-1;
		if (idNodo!=null && duracion!=null && idClase!=null) 
			if (!idNodo.isEmpty() && duracion>=0 && !idClase.isEmpty()) {
				String sql="INSERT INTO PROGRAMACION_TEMPORAL (NODO,CLASE,DURACION)"+
						" VALUES ("+idNodo+","+idClase+","+duracion+");";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+","+idClase+","+duracion+") insertados = "+ejecucion);
	}
	public static void borrarProgramacionDuracionNodo(String idNodo, String idClase){
		int ejecucion=-1;
		if (idNodo!=null && idClase!=null) 
			if (!idNodo.isEmpty() && !idClase.isEmpty()) {
				String sql="DELETE FROM PROGRAMACION_TEMPORAL" +
						" WHERE PROGRAMACION_TEMPORAL.NODO="+idNodo+
						" AND PROGRAMACION_TEMPORAL.CLASE="+idClase+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+","+idClase+") insertados = "+ejecucion);
	}
	public static Vector<Vector<String>> consultarProgramacionDuracionNodo(String idNodo,String idClase){
		Vector<Vector<String>> tabla=null;
		if (idNodo!=null && idClase!=null) {
			if (!idNodo.isEmpty() && !idClase.isEmpty()) {
				String sql="SELECT NODO,CLASE,DURACION FROM PROGRAMACION_TEMPORAL" +
						" WHERE NODO="+idNodo+" AND CLASE="+idClase+";";
				tabla=BBDD.consultaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+","+idClase+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static void borrarProgramacionNodo(String idNodo){
		int ejecucion=-1;
		Vector<Vector<String>> tabla=null;
		if (idNodo!=null) {
			if (!idNodo.equals("")) {
				tabla = BBDD.consultarProgramacionNodoHijos(idNodo,null);
				if (tabla!=null) if (tabla.size()>0) {
					for (int i=0;i<tabla.size();i++){
						borrarProgramacionNodo(tabla.get(i).get(0));
					}
				}
				String sql="DELETE FROM PROGRAMACION_TEMPORAL WHERE PROGRAMACION_TEMPORAL.NODO="+idNodo+";";
				sql+="DELETE FROM PROGRAMACION_NODO WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+") hijos ("+((tabla!=null)?tabla.size():"null")+") borrados "+ejecucion);
	}
	public static void modificarProgramcionNodoBajarNodo(String idNodo, String idNuevoPadre,boolean primero){
		int ejecucion=-1;
		//System.out.println("BBDD modificarProgramcionNodoBajarNodo ("+idNodo+","+idNuevoPadre+","+primero+")");
		Vector<Vector<String>> hermanosPosteriores=null;
		Vector<Vector<String>> hermanosNuevos=null;
		if (idNodo!=null && idNuevoPadre!=null) {
			if (!idNodo.equals("") && !idNuevoPadre.equals("")) {
				Vector<Vector<String>> nodo = BBDD.consultarProgramacionNodo(idNodo);
				String idPadre = nodo.get(0).get(1);
				Integer posNodo = Integer.valueOf(nodo.get(0).get(2));
				//System.out.println("BBDD modificarProgramcionNodoBajarNodo (padre "+idPadre+", posNodo"+posNodo+")");
				// adelantar los hermanos posteriores, si primero poner en 0 y correr los demas hermanos sino el último
				String sql="";
				String sql2="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE"+((idPadre==null)?" IS NULL AND MATERIA="+nodo.get(0).get(3):"="+idPadre)+
						" AND POSICION>"+posNodo+
						" ORDER BY POSICION ASC;";
				hermanosPosteriores=BBDD.consultaSQL(sql2);
				//System.out.println("BBDD modificarProgramcionNodoBajarNodo (hermanos "+hermanosPosteriores.size()+")");
				if (hermanosPosteriores!=null) if (hermanosPosteriores.size()>0) {
					for (int i=0;i<hermanosPosteriores.size();i++){
						sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(i+posNodo)+
						" WHERE PROGRAMACION_NODO.ID="+hermanosPosteriores.get(i).get(0)+";";
					}
				}
				sql2="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE="+idNuevoPadre+" ORDER BY POSICION ASC;";
				hermanosNuevos=BBDD.consultaSQL(sql2);
				//System.out.println("BBDD modificarProgramcionNodoBajarNodo (hermanosNuevos "+hermanosPosteriores.size()+")");
				if (primero){
					//System.out.println("BBDD modificarProgramcionNodoBajarNodo (primero y desplaza los demás)");
					sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION=1" +
							" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
					sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.PADRE="+idNuevoPadre+
							" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
					if (hermanosNuevos!=null) if (hermanosNuevos.size()>0) {
						for (int i=0;i<hermanosNuevos.size();i++){
							sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(i+2)+
									" WHERE PROGRAMACION_NODO.ID="+hermanosNuevos.get(i).get(0)+";";
						}
					}
				} else {
					//System.out.println("BBDD modificarProgramcionNodoBajarNodo (despues del último)");
					sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(hermanosNuevos.size()+1)+
							" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
					sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.PADRE="+idNuevoPadre+
							" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				}
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+")" +" hermanos posteriores ("+((hermanosPosteriores!=null)?hermanosPosteriores.size():"null")+")" +" hermanos posteriores nuevos ("+((hermanosNuevos!=null)?hermanosNuevos.size():"null")+")" +" total modificados "+ejecucion);
	}
	public static void modificarProgramcionNodoSubirNodo(String idNodo){
		int ejecucion=-1;
		Vector<Vector<String>> hermanosPosterioresAntiguos=null;
		Vector<Vector<String>> hermanosPadrePosteriores=null;
		//System.out.println("BBDD modificarProgramcionNodoSubirNodo ("+idNodo+")");
		if (idNodo!=null) {
			if (!idNodo.equals("")) {
				Vector<Vector<String>> nodo = BBDD.consultarProgramacionNodo(idNodo);
				String idPadre = nodo.get(0).get(1);
				Integer posNodo = Integer.valueOf(nodo.get(0).get(2));
				//System.out.println("BBDD modificarProgramcionNodoSubirNodo nodo (padre "+idPadre+", posNodo"+posNodo+")");
				nodo = BBDD.consultarProgramacionNodo(idPadre);
				String idAbuelo = nodo.get(0).get(1);
				Integer posPadre = Integer.valueOf(nodo.get(0).get(2));
				//System.out.println("BBDD modificarProgramcionNodoSubirNodo padre (abuelo "+idAbuelo+", posPadre"+posPadre+")");
				if (idAbuelo==null) idAbuelo="NULL";
				String sql="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.PADRE="+idAbuelo+
						" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+posPadre+
						" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				String sql2="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE="+idPadre+" AND POSICION>"+posNodo+
						" ORDER BY POSICION ASC;";
				hermanosPosterioresAntiguos=BBDD.consultaSQL(sql2);
				//System.out.println("BBDD modificarProgramcionNodoSubirNodo (hermanos "+hermanosPosterioresAntiguos.size()+")");
				if (hermanosPosterioresAntiguos!=null) if (hermanosPosterioresAntiguos.size()>0) {
					for (int i=0;i<hermanosPosterioresAntiguos.size();i++){
						sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(i+posNodo)+
						" WHERE PROGRAMACION_NODO.ID="+hermanosPosterioresAntiguos.get(i).get(0)+";";
					}
				}
				sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(1+posPadre)+
				" WHERE PROGRAMACION_NODO.ID="+idPadre+";";
				sql2="SELECT ID,PADRE,POSICION FROM PROGRAMACION_NODO" +
						" WHERE PADRE"+((idAbuelo.equals("NULL"))?" IS NULL AND MATERIA="+nodo.get(0).get(3):"="+idAbuelo)+
						" AND POSICION>"+posPadre+
						" ORDER BY POSICION ASC;";
				//System.out.println("sql (padre "+posPadre+") "+sql2);
				hermanosPadrePosteriores=BBDD.consultaSQL(sql2);
				//System.out.println("BBDD modificarProgramcionNodoSubirNodo (hermanosPadre "+hermanosPadrePosteriores.size()+")");
				if (hermanosPadrePosteriores!=null) if (hermanosPadrePosteriores.size()>0) {
					for (int i=0;i<hermanosPadrePosteriores.size();i++){
						sql+="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION="+(2+i+posPadre)+
						" WHERE PROGRAMACION_NODO.ID="+hermanosPadrePosteriores.get(i).get(0)+";";
					}
				}
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+")" +" hermanos posteriores antiguos ("+((hermanosPosterioresAntiguos!=null)?hermanosPosterioresAntiguos.size():"null")+")" +" hermanos padre posteriores ("+((hermanosPadrePosteriores!=null)?hermanosPadrePosteriores.size():"null")+")" +" total modificados "+ejecucion);
	}
	public static void modificarProgramcionNodoIntercambiaPos(String idNodoA, String idNodoB){
		int ejecucion=-1;
		if (idNodoA!=null && idNodoB!=null) {
			if (!idNodoA.equals("") && !idNodoB.equals("")) {
				Vector<Vector<String>> nodo = BBDD.consultarProgramacionNodo(idNodoA);
				String posA = nodo.get(0).get(2); 
				nodo = BBDD.consultarProgramacionNodo(idNodoB);
				String posB = nodo.get(0).get(2);
				String sql="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION='"+posB+"'" +
						" WHERE PROGRAMACION_NODO.ID="+idNodoA+";" +
						" UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.POSICION='"+posA+"'" +
						" WHERE PROGRAMACION_NODO.ID="+idNodoB+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodoA+","+idNodoB+") modificados "+ejecucion);
	}
	public static void modificarProgramcionNodoEnunciado(String idNodo, String enunciado){
		int ejecucion=-1;
		if (idNodo!=null && enunciado!=null) {
			if (!idNodo.equals("") && !enunciado.equals("")) {
				String sql="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.ENUNCIADO='"+enunciado+"'"+
				" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+","+enunciado+") modificados "+ejecucion);
	}
	public static Vector<Vector<String>> consultarProgramacionNodo(String idNodo){
		Vector<Vector<String>> tabla=null;
		if (idNodo!=null) {
			if (idNodo!="") {
			String sql="SELECT ID,PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO" +
					" FROM PROGRAMACION_NODO" +
					" WHERE PROGRAMACION_NODO.ID="+idNodo+";";
			tabla=BBDD.consultaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNodo+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarProgramacionNodoHijos(String idPadre,String idMateria){
		Vector<Vector<String>> tabla=null;
		if (idPadre!=null || idMateria!=null) {
			if (idPadre!="") {
			String sql="SELECT ID,PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO FROM PROGRAMACION_NODO" +
					" WHERE "+((idPadre!=null)?"PADRE="+idPadre:"PADRE IS NULL AND MATERIA="+idMateria)+
					" ORDER BY POSICION ASC;";
			tabla=BBDD.consultaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idPadre+","+idMateria+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static void borrarArchivo(String nomArchivo){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+")");
		if (nomArchivo!=null) {
			File file = new File(nomArchivo);
			if (file.exists()) {
				if (file.delete()) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+") borrado el archivo.");
				} else {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+") No pudo borrar el archivo.");
				}
			} else {
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+") No exite el archivo.");
			}
		}
	}
	public static void cerrarBBDD(){
		Connection tConexion = null;
		Statement sentencia = null;
		try {
			tConexion = DriverManager.getConnection(BBDD.cadenaConexion+BBDD.cadenaConexionCierre);
			sentencia = tConexion.createStatement();
			sentencia.execute("SHUTDOWN IMMEDIATELY;");
			tConexion.close();
		} catch (SQLException e) {}
		sentencia = null;
		tConexion = null;
		try {
			tConexion = DriverManager.getConnection(BBDD.cadenaConexion+BBDD.cadenaConexionCierre);
			sentencia = tConexion.createStatement();
			sentencia.execute("SHUTDOWN IMMEDIATELY;");
			tConexion.close();
		} catch (SQLException e) {}
	}
	public static String quitaEscapes(String origen){
		String cadena="";
		String s=null;
		int iu=0;
		if (origen!=null) {
			for (byte b : origen.getBytes()){
				if (s!=null) {
					iu++;
					if (iu>2) s+=(char)b;
					if (iu==6) {
						cadena+=(char)Integer.parseInt( s, 16 );
						s=null;
						iu=0;
					}
				} else {
					if ((char)b=='\\') {
						iu=1;
						s="";
					} else cadena+=(char)b;
				}
				//System.out.println((char)b+":"+iu+"s"+s);
			}
			//System.out.println("origen "+origen+" => "+cadena);
		}
		return cadena;
	}
	public static boolean modificarClave(String acceso){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+acceso+")",false);
		boolean resultado=false;
		int ejecucion=0;
		if (acceso!=null) if (!acceso.isEmpty()) {
			BBDD.copiaSeguridadHacer();
			BBDD.ejecutaSQL("SCRIPT '"+BBDD.archivoDUMP+"';");
			BBDD.cerrarBBDD();
			borrarArchivo(BBDD.archivoLOG);
			borrarArchivo(BBDD.archivoLCK);
			borrarArchivo(BBDD.archivoSCRIPT);
			borrarArchivo(BBDD.archivoPROPERTIES);
			if (BBDD.acceso(acceso)){
				acceso=null;
				BBDD.crear();
		        File archivo = new File(BBDD.archivoDUMP);
		        //File archivo = new File(BBDD.archivoDUMP+".clase");
		        FileReader fr=null;
				try {fr = new FileReader(archivo);
				} catch (FileNotFoundException e1) {e1.printStackTrace();}
		        BufferedReader br = new BufferedReader(fr);
		        String linea=null;
		        Vector<String> sql = new Vector<String>();
		        int nlinea=1;
		        do {
		        	try {
		        		linea=br.readLine();
		        		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("linea "+nlinea+":"+quitaEscapes(linea)+" > ");
		        		if (linea!=null)
		        			if (linea.matches("^INSERT.*"))
		        				if (!linea.matches(".* BLOCKS .*") && !linea.matches(".* TIPOS_EVENTO .*")) {
		        					//System.out.print("ok");
		        					sql.add(quitaEscapes(linea)+";");
		        					//System.out.println(quitaEscapes(linea)+"; = "+BBDD.ejecutaSQL(quitaEscapes(linea)+";"));
		        				}
		        		//System.out.println(".");
		        		
					} catch (IOException e2) {e2.printStackTrace();}
		        	nlinea++;
		        } while (linea != null);
		        if (!sql.isEmpty()) ejecucion=BBDD.ejecutaVariasSQL(sql);
				resultado=true;
			}
			borrarArchivo(BBDD.archivoDUMP);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+acceso+") "+ejecucion+" sentencias sql copiadas.",true);
		return resultado;
	}
	public static void limpiarBBDD(){
		String id = null;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("()");
		Vector<Vector<String>> tabla = BBDD.consultarProfesor(BBDD.consultarIdPropio());
		BBDD.cerrarBBDD();
		borrarArchivo(BBDD.archivoLOG);
		borrarArchivo(BBDD.archivoLCK);
		borrarArchivo(BBDD.archivoSCRIPT);
		borrarArchivo(BBDD.archivoPROPERTIES);
		BBDD.crear();
		if (tabla!=null) if (tabla.size()>0) {
			id = BBDD.insertarProfesor(tabla.get(0).get(1),tabla.get(0).get(2),tabla.get(0).get(3));
			if (id!=null) if (!id.isEmpty()) {
				BBDD.insertarUsuario(id);
			}
		}		
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() usuarioID: "+id+" fin");
	}
	public static void copiaSeguridadRestaurar(String nomArchivo){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+")");
        if (nomArchivo!=null) {
        	borrarArchivo(BBDD.archivoLOG);
        	borrarArchivo(BBDD.archivoLCK);
			FileInputStream fis=null;
			try {fis = new FileInputStream(nomArchivo);
			} catch (FileNotFoundException e1) {e1.printStackTrace();}
			ZipInputStream zis = new ZipInputStream(new BufferedInputStream(fis));
			ZipEntry entry;
			BufferedOutputStream destino=null;
			try {
				while((entry = zis.getNextEntry()) != null) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Extrayendo: " +entry);
				    int cuenta;
				    byte datos[] = new byte[BBDD.bufferArchivo];
				    FileOutputStream fos = new FileOutputStream(entry.getName());
				    destino = new BufferedOutputStream(fos, BBDD.bufferArchivo);
				    while ((cuenta = zis.read(datos, 0, BBDD.bufferArchivo))!=-1) {
				       destino.write(datos, 0, cuenta);
				    }
				    destino.flush();
				    destino.close();
				 }
			} catch (FileNotFoundException e) {e.printStackTrace();
			} catch (IOException e) {e.printStackTrace();}
			try {zis.close();} catch (IOException e) {e.printStackTrace();}
        }
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() fin");
	}
	public static void copiaSeguridadHacer(){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("()");
		FileOutputStream fout=null;
		Calendar fecha = Calendar.getInstance();
		String nombreCopia=PCuaderno.MiPCuaderno.titulo+"_copia_"+Fecha.nomArchivoFecha(fecha)+".zip";
		try {fout = new FileOutputStream(nombreCopia);
		} catch (FileNotFoundException e1) {e1.printStackTrace();}
		ZipOutputStream archivoZIP = new ZipOutputStream(fout);
		archivoZIP.setLevel(9);
		ZipEntry zProperties = new ZipEntry(BBDD.archivoPROPERTIES);
		FileInputStream aProperties=null;
		ZipEntry zScript = new ZipEntry(BBDD.archivoSCRIPT);
		FileInputStream aScript=null;
		try {
			aProperties = new FileInputStream(BBDD.archivoPROPERTIES);
			aScript = new FileInputStream(BBDD.archivoSCRIPT);
		} catch (FileNotFoundException e1) {e1.printStackTrace();}
		try {
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Comprimiendo "+BBDD.archivoPROPERTIES+" ...");
			archivoZIP.putNextEntry(zProperties);
			for (int c=aProperties.read();c!=-1;c=aProperties.read()) {archivoZIP.write(c);}
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Comprimiendo "+BBDD.archivoSCRIPT+" ...");
			archivoZIP.putNextEntry(zScript);
			for (int c=aScript.read();c!=-1;c=aScript.read()) {archivoZIP.write(c);}
		} catch (IOException x) {x.printStackTrace();
		} finally {	try {aProperties.close();} catch (IOException e1) {e1.printStackTrace();}}
		try {archivoZIP.close();
		} catch (IOException e1) {e1.printStackTrace();}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() => "+nombreCopia);
	}
	public static void importarEstudiantes(String nomArchivo,String idClase,boolean cabecera){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nomArchivo+","+idClase+","+cabecera+")");
    	int ejecucion=0;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") archivo "+nomArchivo+" ("+BBDD.conjuntoCaracteresEstudiantes+")");
        if (nomArchivo!=null) {
        	String sql="";
			String linea=null;
			BufferedReader bf=null;
			InputStreamReader isr=null;
			try {
				isr = new InputStreamReader(new FileInputStream(nomArchivo),Charset.forName(BBDD.conjuntoCaracteresEstudiantes));
			} catch (FileNotFoundException e) {e.printStackTrace();}
			bf = new BufferedReader (isr);
			//System.out.println("codificación "+isr.getEncoding());
			int nlinea=0;
			try {
				while ((linea = bf.readLine())!=null) {
					nlinea++;
					if (nlinea>1 || !cabecera) {
						linea=linea.replace("\"", "");
						linea=linea.replace("'", "");
						String[] campos=linea.split(",");
						if (campos.length<2 || campos.length>2) {
							System.out.println("Error: "+linea);
							System.out.println("Error al importar estudiante: linea con ningúna o más de una coma.");
							break;
						}
						campos[0]=campos[0].trim();
						campos[1]=campos[1].trim();
						//System.out.println(campos[1]+" "+campos[0]);
						sql+="INSERT INTO ESTUDIANTES (CLASE,NOMBRE,APELLIDOS) VALUES ("+idClase+",'"+campos[1]+"','"+campos[0]+"');";
					}
				}
				ejecucion=BBDD.ejecutaSQL(sql);
			} catch (IOException e1) {e1.printStackTrace();} 
        }
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") "+ejecucion+" estudiantes importados.");
	}
	public static void importarProgramacion(String idMateria){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+")");
        FileDialog fd = new FileDialog(PCuaderno.MiPCuaderno.shellPrincipal, SWT.OPEN);
        fd.setText("Importar índice desde");
        fd.setFilterPath(BBDD.dirBBDD);
        String[] filterExt = { "*.txt", "*.*" };
        fd.setFilterExtensions(filterExt);
        String nomArchivo = fd.open();
        //System.out.println("BBDD importarProgramacion desde "+nomArchivo);
        int nlinea=0;
        if (nomArchivo!=null) {
	        File archivo = new File(nomArchivo);
	        FileReader fr=null;
			try {fr = new FileReader(archivo);
			} catch (FileNotFoundException e1) {e1.printStackTrace();}
	        BufferedReader br = new BufferedReader(fr);
	        String linea=null;
	        nlinea=1;
	        int nivel=0;
	        int nivelAnterior=0;
	        Vector<String> idPadres = new Vector<String>();idPadres.add("NULL");
	        Vector<Integer> posiciones = new Vector<Integer>();posiciones.add(0);
	        String idNodo=null;
	        //System.out.print("BBDD importarProgramacion:");
	        Connection conexion=abrirConexion();
	        do {
	        	try {
	        		linea=br.readLine();
	        		if (linea!=null) {
	        			nivel=0;
	        			while (linea.charAt(nivel)=='\t') nivel++;
	        			if (nivel==nivelAnterior+1) {
	        				idPadres.add(idNodo);
	        				posiciones.add(1);
	        			} else if (nivel==nivelAnterior) {
	        				Integer ultimo=posiciones.lastElement();
	        				posiciones.set(posiciones.size()-1, ultimo+1);
	        				//System.out.println("pos="+posiciones.lastElement());
	        			} else if (nivel<nivelAnterior) {
	        				for (int i=0;i<nivelAnterior-nivel;i++) {
	        					idPadres.remove(idPadres.size()-1);
	        					posiciones.remove(posiciones.size()-1);
	            				Integer ultimo=posiciones.lastElement();
	            				posiciones.set(posiciones.size()-1, ultimo+1);
	        				}
	        			} else {
	        				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Error importando índice de "+nomArchivo+" en la linea "+nlinea+" bajando más de un nivel.");
	        				break;
	        			}
	        			//System.out.println("linea (nivel="+nivel+" padre="+idPadres.lastElement()+"/"+idPadres.size()+" posiciones"+posiciones.lastElement()+"/"+posiciones.size()+"): "+linea);
	        			//idNodo=BBDD.insertarProgramacionNodo(linea.substring(nivel), ((idPadres.size()==0)?null:idPadres.lastElement()), posiciones.lastElement(), MateriaClase.getIdMateriaDeTrabajo());
	        			String enunciado=linea.substring(nivel);
	        			if (enunciado.length()>256) enunciado=enunciado.substring(0,256);
	    				String sql="INSERT INTO PROGRAMACION_NODO (PADRE,POSICION,MATERIA,ENUNCIADO,SUMARIO)"+
	    						" VALUES ("+((idPadres.size()==0)?null:idPadres.lastElement())+
	    						","+posiciones.lastElement()+","+MateriaClase.getIdMateriaDeTrabajo()+
	    						",'"+enunciado+"','');";
	    				idNodo = BBDD.insertarIdSQLAbierta(conexion,sql,"PROGRAMACION_NODO");
	    				nivelAnterior=nivel;
	        			nlinea++;
	        	        //System.out.print(".");
	        		}
				} catch (IOException e2) {e2.printStackTrace();}
	        } while (linea != null);
	        //System.out.println("");
	        cerrarConexion(conexion);
        }
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("desde "+nomArchivo+" "+nlinea+" lineas con registros insertados.");
	}
	public static void borrarEvento(String idEvento){
		int ejecucion=-1;
		if (idEvento!=null) {
			if (idEvento!="") {
				String sql=null;
				sql="SELECT EVENTOS.TIPO FROM EVENTOS WHERE EVENTOS.ID="+idEvento+";";
				Vector<Vector<String>> respuesta=BBDD.consultaSQL(sql);
				if (respuesta!=null) {
					if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoEvaluacion) {
						sql="DELETE FROM EVALUACIONES WHERE EVALUACIONES.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoClaustro) {
						sql="DELETE FROM CLAUSTROS WHERE CLAUSTROS.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoConsejoEscolar) {
						sql="DELETE FROM C_ESCOLARES WHERE C_ESCOLARES.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoCitaConTutores) {
						sql="DELETE FROM ESTUDIANTES_CITATUTORES WHERE ESTUDIANTES_CITATUTORES.EVENTO_CITATUTORES="+idEvento+";";
						BBDD.ejecutaSQL(sql);
						sql="DELETE FROM CITAS_TUTORES WHERE CITAS_TUTORES.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoETCP) {
						sql="DELETE FROM ETCPS WHERE ETCPS.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoOtroTipo) {
						sql="DELETE FROM OTROS_EVENTOS WHERE OTROS_EVENTOS.ID="+idEvento+";";
					}
				}
				sql+="DELETE FROM EVENTOS WHERE EVENTOS.ID="+idEvento+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvento+") borrados "+ejecucion);
	}
	public static void modificarEstudiantesCitaTutor(String idCita, Vector<String> nuevos){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCita+","+nuevos+")");
		if (idCita!=null && nuevos!=null) if (!idCita.isEmpty()) {
			borrarEstudiantesCitaTutores(idCita);
			for (int i=0;i<nuevos.size();i++){
				insertarEstudianteCitaTutores(idCita,nuevos.get(i));
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCita+","+nuevos+") fin");
	}
	public static String insertarEstudiante(String idClase,String nombre,String apellidos){
		String id=null;
		if (idClase!=null && nombre!=null && apellidos!=null) {
			if (!idClase.isEmpty() && !nombre.isEmpty() && !apellidos.isEmpty()) {
				String sql="INSERT INTO ESTUDIANTES (CLASE,NOMBRE,APELLIDOS) VALUES ("+idClase+",'"+nombre+"','"+apellidos+"');";
				id=BBDD.insertarIdSQL("ESTUDIANTES", sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+","+nombre+","+apellidos+") id del insertado "+id);
		return id;
	}
	public static void borrarEstudiante(String idEstudiante){
		int ejecucion=-1;
		Vector<Vector<String>> citasTutores=null;
		Vector<Vector<String>> evalEvaluable=null;
		if (idEstudiante!=null) if (!idEstudiante.equals("")) {
			String sql="DELETE FROM ESTUDIANTES_CITATUTORES" +
					" WHERE ESTUDIANTES_CITATUTORES.ESTUDIANTE="+idEstudiante+";";
			citasTutores = BBDD.consultaSQL("SELECT EVENTO_CITATUTORES FROM ESTUDIANTES_CITATUTORES" +
					" WHERE ESTUDIANTES_CITATUTORES.ESTUDIANTE="+idEstudiante+";");
			if (citasTutores!=null) if (citasTutores.size()>0) {
				for (int i=0;i<citasTutores.size();i++) {
					Vector<Vector<String>> citas=BBDD.consultarEstudiantesCitaTutores(citasTutores.get(i).get(0));
					if (citas!=null) if (citas.size()==1) {
						sql+="DELETE FROM CITAS_TUTORES WHERE CITAS_TUTORES.ID="+citasTutores.get(i).get(0)+";";
						sql+="DELETE FROM EVENTOS WHERE EVENTOS.ID="+citasTutores.get(i).get(0)+";";
					}
				}
			}
			evalEvaluable = BBDD.consultaSQL("SELECT EVALUACIONES_EVALUABLES.ID,EVALUACIONES_EVALUABLES.GRUPO" +
					" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
					" WHERE EVALUACIONES_EVALUABLES.ID=EE_ESTUDIANTES.E_EVALUABLE" +
					" AND EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante+";");
			sql+="DELETE FROM EE_ESTUDIANTES WHERE EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante+";";
			if (evalEvaluable!=null) if (evalEvaluable.size()>0) {
				for (int i=0;i<evalEvaluable.size();i++) {
					if (evalEvaluable.get(i).get(1).isEmpty()) 
						sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE EVALUACIONES_EVALUABLES.ID="+evalEvaluable.get(i).get(0)+";";
					else {
						Vector<Vector<String>> estudiantesGrupo=BBDD.consultarEstudiantesEvalEvaluable(evalEvaluable.get(i).get(0));
						if (estudiantesGrupo!=null) if (estudiantesGrupo.size()==1)
							sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE EVALUACIONES_EVALUABLES.ID="+evalEvaluable.get(i).get(0)+";";
					}
				}
			}
			sql+="DELETE FROM ESTUDIANTE_MATERIACLASE WHERE ESTUDIANTE_MATERIACLASE.ESTUDIANTE="+idEstudiante+";";
			sql+="DELETE FROM ESTUDIANTES WHERE ESTUDIANTES.ID="+idEstudiante+";";
			ejecucion=BBDD.ejecutaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+") evalEvaluable ("+((evalEvaluable!=null)?evalEvaluable.size():"null")+") citasTutores ("+((citasTutores!=null)?citasTutores.size():"null")+") total borrados "+ejecucion);
	}
	public static void insertarEstudianteCitaTutores(String idCita,String idEstudiante){
		int ejecucion=-1;
		if (idCita!=null && idEstudiante!=null) {
			if (!idCita.isEmpty() && !idEstudiante.isEmpty()) {
				String sql="INSERT INTO ESTUDIANTES_CITATUTORES (ESTUDIANTE,EVENTO_CITATUTORES)"+
				" VALUES ("+idEstudiante+","+idCita+");";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCita+","+idEstudiante+") insertados "+ejecucion);
	}
	public static void borrarEstudiantesCitaTutores(String idCita){
		int ejecucion=-1;
		if (idCita!=null) if (idCita!="") {
				String sql="DELETE FROM ESTUDIANTES_CITATUTORES WHERE EVENTO_CITATUTORES="+idCita+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCita+") borrados "+ejecucion);
	}
	public static void modificarClaseEvaluacion(String idEvaluacion, String idClase){
		int ejecucion=-1;
		if (idEvaluacion!=null && idClase!=null) {
			if (!idEvaluacion.equals("") && !idClase.equals("")) {
				String sql="UPDATE EVALUACIONES SET EVALUACIONES.CLASE="+idClase+
				" WHERE EVALUACIONES.ID="+idEvaluacion+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluacion+","+idClase+") modificados "+ejecucion);
	}
	public static void modificarClaseTutor(String idClase, String idTutor){
		int ejecucion=-1;
		if (idClase!=null && idTutor!=null) {
			if (!idClase.equals("") && !idTutor.equals("")) {
				String sql="UPDATE CLASES SET CLASES.TUTOR="+idTutor+
				" WHERE CLASES.ID="+idClase+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+","+idTutor+") modificados "+ejecucion);
	}
	public static void modificarOrdinarioEvento(String idEvento, boolean Ordinario){
		int ejecucion=-1;
		if (idEvento!=null) {
			if (!idEvento.equals("")) {
				String sql=null;
				sql="SELECT EVENTOS.TIPO FROM EVENTOS WHERE EVENTOS.ID="+idEvento+";";
				Vector<Vector<String>> respuesta=BBDD.consultaSQL(sql);
				if (respuesta!=null) {
					if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoClaustro) {
						sql="UPDATE CLAUSTROS SET CLAUSTROS.ORDINARIO='"+String.valueOf(Ordinario).toUpperCase()+"'"+
						" WHERE CLAUSTROS.ID="+idEvento+";";
					} else if (Integer.parseInt(respuesta.get(0).get(0))==BBDD.idTipoConsejoEscolar) {
						sql="UPDATE C_ESCOLARES SET C_ESCOLARES.ORDINARIO='"+String.valueOf(Ordinario).toUpperCase()+"'"+
						" WHERE C_ESCOLARES.ID="+idEvento+";";
					}
				}
				//System.out.println("sql "+sql);
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvento+","+Ordinario+") modificados "+ejecucion);
	}
	public static void ejecutaSqlUpdateTextDemorado() {
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() texto="+TextoDemorado.clase.ctrl.getText());
		String sql=null;
		int ejecucion=-1;
		if (TextoDemorado.clase!=null && TextoDemorado.sqlId1!=null) {
			if (PCuaderno.MiPCuaderno.evento.getTextAreaSumario()==TextoDemorado.clase.ctrl){
				sql="UPDATE EVENTOS SET EVENTOS.SUMARIO='"+BBDD.sqlDemoradoINFO+"' WHERE EVENTOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextAreaPreparativos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVENTOS SET EVENTOS.PREPARATIVOS='"+BBDD.sqlDemoradoINFO+"' WHERE EVENTOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextAreaConclusion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVENTOS SET EVENTOS.CONCLUSION='"+BBDD.sqlDemoradoINFO+"' WHERE EVENTOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextCitaTutoresRelacion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE CITAS_TUTORES SET CITAS_TUTORES.RELACION='"+BBDD.sqlDemoradoINFO+"' WHERE CITAS_TUTORES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				PCuaderno.MiPCuaderno.calendario.actualizarEtiquetaDia(PCuaderno.MiPCuaderno.evento.getFechaComboFecha().fechaSeleccionada());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextAreaEvaluacionDescripcion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUACIONES SET EVALUACIONES.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUACIONES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				PCuaderno.MiPCuaderno.calendario.actualizarEtiquetaDia(PCuaderno.MiPCuaderno.evento.getFechaComboFecha().fechaSeleccionada());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evento.getTextAreaOtroDescripcion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE OTROS_EVENTOS SET OTROS_EVENTOS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE OTROS_EVENTOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextReferencia()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUABLES SET EVALUABLES.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUABLES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.evaluable.actualizarSoloListaEvaluables();
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextPeso()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUABLES SET EVALUABLES.PESO='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUABLES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextAreaEnunciado()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUABLES SET EVALUABLES.ENUNCIADO='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUABLES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextAreaEvaluacionesObservaciones()==TextoDemorado.clase.ctrl){
				sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.OBSERVACIONES='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUACIONES_EVALUABLES.ID="+BBDD.consultarIdEvalEstudianteEvaluable(TextoDemorado.sqlId1,TextoDemorado.sqlId2)+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextEvaluacionesNota()==TextoDemorado.clase.ctrl){
					String idEval=BBDD.consultarIdEvalEstudianteEvaluable(TextoDemorado.sqlId1, TextoDemorado.sqlId2);
					Point posicion=TextoDemorado.clase.ctrl.getSelection();
					if (TextoDemorado.clase.ctrl.getText().equals("")){
						if (idEval!=null) {
							Vector<Vector<String>> evaluacion = BBDD.consultarEval(idEval);
							//System.out.println("ejecutaSqlDemorado idEval = "+idEval+" grupo="+evaluacion.get(0).get(4));
							if (evaluacion.get(0).get(4).equals("")){ // Si no es grupo
								//System.out.println("borrando");
								BBDD.borrarEvalEvaluable(idEval);
								PCuaderno.MiPCuaderno.evaluable.actualizarNotaObservaciones();
								TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(0);
							} else { // Si pertenece a un grupo 
								sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.NOTA='"+BBDD.sqlDemoradoINFO+
									"' WHERE EVALUACIONES_EVALUABLES.ID="+idEval+";";
								//System.out.println("Inactividad Nota: "+sql+" | "+TextoDemorado.clase.ctrl.getText());
								ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,"0");
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
								TextoDemorado.modificaSinActualizar(TextoDemorado.clase.ctrl,"0");
								TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(1);
							}
						}
					} else {
						//String nota=Numero.validarNumeroDecimal(TextoDemorado.clase.ctrl.getText(),Numero.notaMax,true);
						String nota=TextoDemorado.clase.ctrl.getText();
						if (nota!=null) {
							if (idEval==null) {
								sql="INSERT INTO EVALUACIONES_EVALUABLES (EVALUABLE,NOTA,OBSERVACIONES,GRUPO)" +
										" VALUES ("+TextoDemorado.sqlId2+","+BBDD.sqlDemoradoINFO+",'','');";
								sql=sql.replaceFirst(BBDD.sqlDemoradoINFO, String.valueOf(Numero.numeroDecimal(nota)));
								//System.out.println("Inactividad Nota1: "+sql+" | "+TextoDemorado.clase.ctrl.getText());
								String id=BBDD.insertarIdSQL("EVALUACIONES_EVALUABLES", sql);					
								sql="INSERT INTO EE_ESTUDIANTES (E_EVALUABLE,ESTUDIANTE)" +
										" VALUES ("+BBDD.sqlDemoradoINFO+","+TextoDemorado.sqlId1+");";
								//System.out.println("Inactividad Nota2: "+sql+" | "+id);
								ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,id);
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" () insertando evaluacion y evaluacion_estudiante. "+TextoDemorado.clase.ctrl.getText());
								PCuaderno.MiPCuaderno.evaluable.actualizarNotaObservaciones();
								TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(1);
							} else {
								sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.NOTA='"+BBDD.sqlDemoradoINFO+
										"' WHERE EVALUACIONES_EVALUABLES.ID="+idEval+";";
								//System.out.println("Inactividad Nota: "+sql+" | "+TextoDemorado.clase.ctrl.getText());
								ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,String.valueOf(Numero.numeroDecimal(nota)));
								if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
							}
						}
					}
					TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.evaluable.getTextEvaluacionGrupoNombre()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.GRUPO='"+BBDD.sqlDemoradoINFO+"' WHERE EVALUACIONES_EVALUABLES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.evaluable.actualizarGrupoEvaluacion();
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtDiaNoLectivoDescripcion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE DIASNOLECTIVOS SET DIASNOLECTIVOS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE DIASNOLECTIVOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Vector<String> noLectivo = BBDD.consultarDiaNoLectivo(TextoDemorado.sqlId1);
				Calendar f = Fecha.fecha(noLectivo.get(2), noLectivo.get(3), noLectivo.get(4));
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.calendario.actualizarEtiquetaDia(f);
				PCuaderno.MiPCuaderno.mantenimiento.actualizarDiasNoLectivosLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtTiposEvaluablesDescripcion()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TIPOS_EVALUABLE SET TIPOS_EVALUABLE.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE TIPOS_EVALUABLE.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtTiposEvaluablesReferencia()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TIPOS_EVALUABLE SET TIPOS_EVALUABLE.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE TIPOS_EVALUABLE.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.mantenimiento.actualizarTiposEvaluablesLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtTiposEvaluablesPeso()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TIPOS_EVALUABLE SET TIPOS_EVALUABLE.PESO='"+BBDD.sqlDemoradoINFO+"' WHERE TIPOS_EVALUABLE.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.mantenimiento.actualizarTiposEvaluablesLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtTiposEvaluablesMinimo()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TIPOS_EVALUABLE SET TIPOS_EVALUABLE.MINIMO='"+BBDD.sqlDemoradoINFO+"' WHERE TIPOS_EVALUABLE.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,String.valueOf(Numero.numeroDecimal(TextoDemorado.clase.ctrl.getText())));
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.programacion.getTextSumario()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE PROGRAMACION_NODO SET PROGRAMACION_NODO.SUMARIO='"+BBDD.sqlDemoradoINFO+"' WHERE PROGRAMACION_NODO.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.programacion.getTextDuracion()==TextoDemorado.clase.ctrl){
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				String duracion=null;
				Vector<Vector<String>> temporal = BBDD.consultarProgramacionDuracionNodo(TextoDemorado.sqlId1, TextoDemorado.sqlId2);
				if (TextoDemorado.clase.ctrl.getText().equals("")){
					if (temporal!=null) if (temporal.size()>0){
						BBDD.borrarProgramacionDuracionNodo(temporal.get(0).get(0), temporal.get(0).get(1));
					}
					duracion="0";
				} else {
					if (temporal!=null) if (temporal.size()>0){
						sql="UPDATE PROGRAMACION_TEMPORAL SET PROGRAMACION_TEMPORAL.DURACION='"+BBDD.sqlDemoradoINFO+"' WHERE" +
								" PROGRAMACION_TEMPORAL.NODO="+TextoDemorado.sqlId1+" AND PROGRAMACION_TEMPORAL.CLASE="+TextoDemorado.sqlId2+";";
					} else {					
						sql="INSERT INTO PROGRAMACION_TEMPORAL (NODO,CLASE,DURACION)" +
								" VALUES ("+TextoDemorado.sqlId1+","+TextoDemorado.sqlId2+","+BBDD.sqlDemoradoINFO+");";
					}
					duracion=Numero.validarNumeroEntero(TextoDemorado.clase.ctrl.getText(),null,false);
					ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,String.valueOf(Numero.numeroEntero(duracion)));
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				}
				PCuaderno.MiPCuaderno.programacion.getTreeConIdIndice().actualizaDuracion(
						PCuaderno.MiPCuaderno.programacion.getTreeConIdIndice().getNodo(TextoDemorado.sqlId1)
						,Numero.numeroEntero(duracion));
				PCuaderno.MiPCuaderno.programacion.actualizarTerminaNodo();
				PCuaderno.MiPCuaderno.programacion.actualizarPorImpartir();
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEnsenanzaRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ENSENANZAS SET ENSENANZAS.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE ENSENANZAS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarEnsenanzas(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEnsenanza()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ENSENANZAS SET ENSENANZAS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE ENSENANZAS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtTituloRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TITULOS SET TITULOS.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE TITULOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarTitulos(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtTitulo()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE TITULOS SET TITULOS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE TITULOS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtNivelRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE NIVELES SET NIVELES.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE NIVELES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarNiveles(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtClaseRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE CLASES SET CLASES.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE CLASES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarClases(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtMateriaRef()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE MATERIAS SET MATERIAS.REFERENCIA='"+BBDD.sqlDemoradoINFO+"' WHERE MATERIAS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarMaterias(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtMateria()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE MATERIAS SET MATERIAS.DESCRIPCION='"+BBDD.sqlDemoradoINFO+"' WHERE MATERIAS.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEstudianteNombre()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ESTUDIANTES SET ESTUDIANTES.NOMBRE='"+BBDD.sqlDemoradoINFO+"' WHERE ESTUDIANTES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarEstudiantes(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEstudianteApellidos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ESTUDIANTES SET ESTUDIANTES.APELLIDOS='"+BBDD.sqlDemoradoINFO+"' WHERE ESTUDIANTES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.matricula.actualizarEstudiantes(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.matricula.getTxtEstudianteDatos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE ESTUDIANTES SET ESTUDIANTES.DATOS='"+BBDD.sqlDemoradoINFO+"' WHERE ESTUDIANTES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtProfesorNombre()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE PROFESORES SET PROFESORES.NOMBRE='"+BBDD.sqlDemoradoINFO+"' WHERE PROFESORES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.mantenimiento.actualizarProfesoresLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtProfesorApellidos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE PROFESORES SET PROFESORES.APELLIDOS='"+BBDD.sqlDemoradoINFO+"' WHERE PROFESORES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
				Point posicion=TextoDemorado.clase.ctrl.getSelection();
				PCuaderno.MiPCuaderno.mantenimiento.actualizarProfesoresLista(null);
				TextoDemorado.clase.ctrl.setFocus();TextoDemorado.clase.ctrl.setSelection(posicion);
			} else if (PCuaderno.MiPCuaderno.mantenimiento.getTxtProfesorDatos()==TextoDemorado.clase.ctrl){ 
				sql="UPDATE PROFESORES SET PROFESORES.DATOS='"+BBDD.sqlDemoradoINFO+"' WHERE PROFESORES.ID="+TextoDemorado.sqlId1+";";
				ejecucion=ejecutaSQLreemplazando(sql,BBDD.sqlDemoradoINFO,TextoDemorado.clase.ctrl.getText());
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() modificados="+ejecucion+" texto="+TextoDemorado.clase.ctrl.getText());
			}
		}
	}
	public static int ejecutaSQLreemplazando(String sql,String exp,String valor){
		sql=sql.replaceFirst(exp, valor);
		//System.out.println("Inactividad: "+sql);
		return BBDD.ejecutaSQL(sql);
	}
	public static void modificarHoraEvento(String idEvento, Calendar hora){
		int ejecucion=-1;
		if (idEvento!=null && hora!=null) {
			if (!idEvento.equals("")) {
				String sql="UPDATE EVENTOS SET EVENTOS.HORA='"+Fecha.bbddSoloHora(hora)+"'"+
				" WHERE EVENTOS.ID="+idEvento+";";
				//System.out.println("sql "+sql);
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvento+","+((hora!=null)?Fecha.formaFechaDiaHora(hora):"null")+") modificados "+ejecucion);
	}
	public static void modificarFechaHoraEvaluable(String idEvaluable, Calendar Fi, String idUhI, Calendar Ff, String idUhF){
		int ejecucion=-1;
		if (Fi!=null&&idUhI!=null&&Ff!=null&&idUhF!=null) {
			if (!idUhI.equals("") && !idUhF.equals("")) {
				String sql="UPDATE EVALUABLES SET EVALUABLES.DIA_INICIO='"+Fecha.bbddSoloDia(Fi)+"'"+
				", EVALUABLES.UH_INICIO="+idUhI+
				", EVALUABLES.DIA_FIN='"+Fecha.bbddSoloDia(Ff)+"'"+
				", EVALUABLES.UH_FIN="+idUhF+
				" WHERE EVALUABLES.ID="+idEvaluable+";";
				//System.out.println("sql "+sql);
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+","+Fecha.formaFecha(Fi)+","+idUhI+","+Fecha.formaFecha(Ff)+","+idUhF+") modificados "+ejecucion);
	}
	public static void modificarFechaEvento(String idEvento, Calendar fecha){
		int ejecucion=-1;
		if (idEvento!=null && fecha!=null) {
			if (!idEvento.equals("")) {
				String sql="UPDATE EVENTOS SET EVENTOS.DIA='"+Fecha.bbddSoloDia(fecha)+"'"+
				" WHERE EVENTOS.ID="+idEvento+";";
				//System.out.println("sql "+sql);
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvento+","+Fecha.formaFecha(fecha)+") modificados "+ejecucion);
	}
	public static String insertarEvento(Integer idTipo,Calendar fecha){
		String id=null;
		if (idTipo!=null) {
			if (idTipo>=0) {
				String sql="INSERT INTO EVENTOS (DIA,HORA,SUMARIO,PREPARATIVOS,CONCLUSION,TIPO)"+
				" VALUES ('"+Fecha.bbddSoloDia(fecha)+"','12:00:00','','','',"+idTipo+");";
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Insertando Evento ("+idTipo+")");
				id = BBDD.insertarIdSQL("EVENTOS",sql);
				if (id!=null) if (!id.equals("")) {
					if (idTipo==BBDD.idTipoEvaluacion) {
						String clase=BBDD.consultarIdClaseTutoria(BBDD.consultarIdPropio());
						sql="INSERT INTO EVALUACIONES (ID,DESCRIPCION,CLASE) VALUES ("+id+",'',"+clase+");";
					} else if (idTipo==BBDD.idTipoClaustro) {
						sql="INSERT INTO CLAUSTROS (ID,ORDINARIO) VALUES ("+id+",true);";
					} else if (idTipo==BBDD.idTipoConsejoEscolar) {
						sql="INSERT INTO C_ESCOLARES (ID,ORDINARIO) VALUES ("+id+",true);";
					} else if (idTipo==BBDD.idTipoCitaConTutores) {
						sql="INSERT INTO CITAS_TUTORES (ID,RELACION) VALUES ("+id+",'');";
					} else if (idTipo==BBDD.idTipoETCP) {
						sql="INSERT INTO ETCPS (ID) VALUES ("+id+");";
					} else if (idTipo==BBDD.idTipoOtroTipo) {
						sql="INSERT INTO OTROS_EVENTOS (ID,DESCRIPCION) VALUES ("+id+",'');";
					}
					BBDD.ejecutaSQL(sql);
				} else if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("No pudo insertar Evento.");
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTipo+") id del insertado = "+id);
		return id;
	}
	public static String insertarEvaluable(String tipo){
		String id=null;
		if (tipo!=null) if (!tipo.isEmpty()) {
			Calendar fecha = (Calendar) Fecha.fechaDeTrabajo.clone();
			fecha=Fecha.getFechaHoraLaboralOSiguiente(fecha, true);
			String sql="INSERT INTO EVALUABLES (DIA_INICIO,UH_INICIO,DIA_FIN,UH_FIN,REFERENCIA,ENUNCIADO,PESO,TIPO)"+
			" VALUES ('"+Fecha.bbddSoloDia(fecha)+"',"+BBDD.consultarIdUHoraria(fecha)+",'"+
			Fecha.bbddSoloDia(fecha)+"',"+BBDD.consultarIdUHoraria(fecha)+",'','',1,"+tipo+");";
			id = BBDD.insertarIdSQL("EVENTOS",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+tipo+") id del insertado = "+id);
		return id;
	}
	public static Vector<Vector<String>> consultarEstudiantesTutoriaProfesor (String idProfesor){
		Vector<Vector<String>> tabla=null;
		if (idProfesor!=null) if (!idProfesor.isEmpty()) {
			String sql="SELECT DISTINCT ESTUDIANTES.ID,ESTUDIANTES.NOMBRE,ESTUDIANTES.APELLIDOS" +
					" FROM ESTUDIANTES,CLASES" +
					" WHERE ESTUDIANTES.CLASE=CLASES.ID" +
					" AND CLASES.TUTOR="+idProfesor + 
					" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idProfesor+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static String consultarIdClaseTutoria (String idProfesor){
		String resultado=null;
		Vector<Vector<String>> tabla=null;
		String sql="SELECT CLASES.ID FROM CLASES" +
				" WHERE CLASES.TUTOR="+idProfesor+";";
		tabla=BBDD.consultaSQL(sql);
		if (tabla!=null) if (tabla.size()>0) resultado=tabla.get(0).get(0);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+resultado);
		return resultado;
	}
	public static String consultarIdPropio (){
		String id=null;
		Vector<Vector<String>> usuario=null;
		String sql="SELECT * FROM USUARIO;";
		usuario=BBDD.consultaSQL(sql);
		if (usuario!=null) if (usuario.size()>0) id=usuario.get(0).get(0);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+id);
		return id;
	}
	public static void insertarUsuario(String id){
		int ejecucion=0;
		if (id!=null) 
			if (!id.isEmpty())
				if (BBDD.consultarIdPropio()==null){
					String sql="INSERT INTO USUARIO (PROFESOR)"+
						" VALUES ("+id+");";
					ejecucion = BBDD.ejecutaSQL(sql);
				}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+") "+ejecucion+" insertardos");
	}
	public static String insertarProfesor(String nombre, String apellidos, String datos){
		String id=null;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nombre+","+apellidos+")");
		if (nombre!=null && apellidos!=null && apellidos!=null) 
			if (!nombre.isEmpty()) {
				String sql="INSERT INTO PROFESORES (NOMBRE,APELLIDOS,DATOS)"+
			" VALUES ('"+nombre+"','"+apellidos+"','"+datos+"');";
			id = BBDD.insertarIdSQL("PROFESORES",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nombre+","+apellidos+") id del insertado = "+id);
		return id;
	}
	public static void borrarProfesor(String idProfesor){
		int ejecucion=-1;
		if (idProfesor!=null) {
			if (!idProfesor.equals("")) {
				String claseTutoria = BBDD.consultarIdClaseTutoria(idProfesor);
				if (claseTutoria==null) {
					String sql="DELETE FROM PROFESORES WHERE PROFESORES.ID="+idProfesor+";";
					ejecucion=BBDD.ejecutaSQL(sql);
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idProfesor+") borrados "+ejecucion);
	}
	public static Vector<Vector<String>> consultarProfesor(String id){
		Vector<Vector<String>> tabla=null;
		String sql="SELECT PROFESORES.ID,PROFESORES.NOMBRE,PROFESORES.APELLIDOS,PROFESORES.DATOS,PROFESORES.ID=USUARIO.PROFESOR"+
				" FROM PROFESORES, USUARIO"+
				((id!=null)?" WHERE PROFESORES.ID="+id:"")+
				" ORDER BY PROFESORES.APELLIDOS,PROFESORES.NOMBRE;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEventosTipoDia(String tipo,Calendar fecha){
		Vector<Vector<String>> tabla; // si fecha=null => cualquier fecha y tipo=null => cualquier tipo 
		String sql="SELECT EVENTOS.ID,DAYOFMONTH(EVENTOS.DIA),MONTH(EVENTOS.DIA),YEAR(EVENTOS.DIA)," +
				" HOUR(EVENTOS.HORA),MINUTE(EVENTOS.HORA), EVENTOS.SUMARIO,EVENTOS.PREPARATIVOS," +
				" EVENTOS.CONCLUSION,TIPOS_EVENTO.ID,TIPOS_EVENTO.DESCRIPCION,TIPOS_EVENTO.COLOR " +
				" FROM EVENTOS, TIPOS_EVENTO" +
				" WHERE EVENTOS.TIPO=TIPOS_EVENTO.ID" +
				((tipo!=null)?((Integer.parseInt(tipo)!=0)?" AND EVENTOS.TIPO="+tipo:""):"")+
				" AND EVENTOS.DIA='"+((fecha!=null)?Fecha.bbddSoloDia(fecha):Fecha.bbddSoloDia(Fecha.inicioCurso))+"'"+
				" ORDER BY EVENTOS.DIA,EVENTOS.HORA;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+tipo+","+Fecha.formaFecha(fecha)+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEventosTipoDesdeFecha(String tipo,Calendar fecha){
		Vector<Vector<String>> tabla; // si fecha=null => cualquier fecha y tipo=null => cualquier tipo 
		String sql="SELECT EVENTOS.ID,DAYOFMONTH(EVENTOS.DIA),MONTH(EVENTOS.DIA),YEAR(EVENTOS.DIA)," +
				" HOUR(EVENTOS.HORA),MINUTE(EVENTOS.HORA), EVENTOS.SUMARIO,EVENTOS.PREPARATIVOS," +
				" EVENTOS.CONCLUSION,TIPOS_EVENTO.ID,TIPOS_EVENTO.DESCRIPCION,TIPOS_EVENTO.COLOR " +
				" FROM EVENTOS, TIPOS_EVENTO" +
				" WHERE EVENTOS.TIPO=TIPOS_EVENTO.ID" +
				((tipo!=null)?((Integer.parseInt(tipo)!=0)?" AND EVENTOS.TIPO="+tipo:""):"")+
				" AND EVENTOS.DIA>='"+((fecha!=null)?Fecha.bbddSoloDia(fecha):Fecha.bbddSoloDia(Fecha.inicioCurso))+"'"+
				" ORDER BY EVENTOS.DIA,EVENTOS.HORA;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+tipo+","+Fecha.formaFecha(fecha)+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarTiposEvento(){
		Vector<Vector<String>> tabla;
		String sql="SELECT ID,DESCRIPCION,COLOR,BBDD FROM TIPOS_EVENTO;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEstudiantesCitaTutores(String idCitaTutores){
		Vector<Vector<String>> tabla=null;
		if (idCitaTutores!=null) {
			String sql="SELECT ESTUDIANTES.ID,ESTUDIANTES.NOMBRE,ESTUDIANTES.APELLIDOS" +
					" FROM ESTUDIANTES_CITATUTORES,ESTUDIANTES" +
					" WHERE ESTUDIANTES_CITATUTORES.ESTUDIANTE=ESTUDIANTES.ID" +
					" AND ESTUDIANTES_CITATUTORES.EVENTO_CITATUTORES="+idCitaTutores +
					" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCitaTutores+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<String> consultarIdEstudiantesCitaTutores(String idCitaTutores) {
		Vector<String> registro=null;
		if (idCitaTutores!=null) {
			String sql="SELECT ESTUDIANTES.ID" +
					" FROM ESTUDIANTES_CITATUTORES,ESTUDIANTES" +
					" WHERE ESTUDIANTES_CITATUTORES.ESTUDIANTE=ESTUDIANTES.ID" +
					" AND ESTUDIANTES_CITATUTORES.EVENTO_CITATUTORES="+idCitaTutores +
					" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
			registro=BBDD.consulta1ColumnaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idCitaTutores+") "+((registro!=null)?registro.size():"null")+" datos");
		return registro;
	}
	public static Vector<Vector<String>> consultarEventoTipoDatos(String id, int tipo){
		Vector<Vector<String>> tabla=null;
		String sql=null;
		//System.out.println("Evento tipo="+tipo+", id="+id);
		if (id!=null) if (!id.isEmpty() && tipo>=0) {
			if (tipo==BBDD.idTipoEvaluacion) {
				sql="SELECT EVALUACIONES.ID,EVALUACIONES.DESCRIPCION, CLASES.ID, NIVELES.REFERENCIA," +
						" CLASES.REFERENCIA,TITULOS.REFERENCIA,PROFESORES.APELLIDOS,PROFESORES.NOMBRE" +
						" FROM EVALUACIONES,CLASES,PROFESORES,NIVELES,TITULOS" +
						" WHERE EVALUACIONES.ID="+id+" AND CLASES.ID=EVALUACIONES.CLASE" +
						" AND CLASES.TUTOR=PROFESORES.ID AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID;";
			} else if (tipo==BBDD.idTipoClaustro) {
				sql="SELECT ID,ORDINARIO FROM CLAUSTROS WHERE ID="+id+";";
			} else if (tipo==BBDD.idTipoConsejoEscolar) {
				sql="SELECT ID,ORDINARIO FROM C_ESCOLARES WHERE ID="+id+";";
			} else if (tipo==BBDD.idTipoCitaConTutores) {
				sql="SELECT ID,RELACION FROM CITAS_TUTORES WHERE CITAS_TUTORES.ID="+id+";";
			} else if (tipo==BBDD.idTipoETCP) {
				sql="SELECT ID FROM ETCPS WHERE ID="+id+";";
			} else if (tipo==BBDD.idTipoOtroTipo) {
				sql="SELECT ID,DESCRIPCION FROM OTROS_EVENTOS WHERE ID="+id+";";
			}
			//System.out.println("sql="+sql);
			if (sql!=null)tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+","+tipo+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEvento(String id){
		Vector<Vector<String>> tabla=null;
		if (id!=null) if (!id.isEmpty()) {
			String sql="SELECT EVENTOS.ID,DAYOFMONTH(EVENTOS.DIA),MONTH(EVENTOS.DIA),YEAR(EVENTOS.DIA)," +
					" HOUR(EVENTOS.HORA),MINUTE(EVENTOS.HORA), EVENTOS.SUMARIO,EVENTOS.PREPARATIVOS," +
					" EVENTOS.CONCLUSION,TIPOS_EVENTO.ID,TIPOS_EVENTO.DESCRIPCION,TIPOS_EVENTO.COLOR " +
					" FROM EVENTOS, TIPOS_EVENTO" +
					" WHERE EVENTOS.TIPO=TIPOS_EVENTO.ID" +
					" AND EVENTOS.ID="+id+
					" ORDER BY EVENTOS.DIA,EVENTOS.HORA;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static boolean consultarUHvacio(String idMateriaClase){
		boolean resultado=true;
		Vector<Vector<String>> tabla=null;
		String sql="SELECT ID FROM UH_SEMANA" +
				((idMateriaClase!=null)?" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase:"")+
				" LIMIT 1;";
		tabla=BBDD.consultaSQL(sql);	
		if (tabla!=null) if (tabla.size()>0) resultado=false;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+") "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static Vector<Vector<String>> consultarUHOrdenadaMateriaClase(String idMateriaClase,int diaSemana){
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null) if (!idMateriaClase.isEmpty() && diaSemana>=0) {
			String sql="SELECT UH_SEMANA.ID,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO),HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN) FROM UH_SEMANA" +
					" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase+" AND UH_SEMANA.DIA_SEMANA="+diaSemana+" ORDER BY UH_SEMANA.INICIO;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+diaSemana+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
		
	}
	public static Vector<Vector<String>> consultarUHMateriaClaseDiaDesdeHora(String idMateriaClase,Calendar fecha){
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null && fecha!=null) if (!idMateriaClase.isEmpty()) {
			String sql="SELECT UH_SEMANA.ID,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO)," +
					"HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN) FROM UH_SEMANA" +
					" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase+
					" AND UH_SEMANA.DIA_SEMANA="+fecha.get(Calendar.DAY_OF_WEEK)+
					" AND UH_SEMANA.FIN>'"+Fecha.bbddSoloHora(fecha)+"'"+
					" ORDER BY UH_SEMANA.INICIO;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+Fecha.formaFechaDiaHora(fecha)+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarUHMateriaClase(String idMateria, String idClase){
		Vector<Vector<String>> tabla=null;
		String sql="SELECT UH_SEMANA.ID,UH_SEMANA.MATERIACLASE,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO)," +
				"HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN),UH_SEMANA.DIA_SEMANA FROM UH_SEMANA,MATERIA_CLASE" +
				" WHERE UH_SEMANA.MATERIACLASE=MATERIA_CLASE.ID"+
				((idMateria!=null)?" AND MATERIA_CLASE.MATERIA="+idMateria:"")+
				((idClase!=null)?" AND MATERIA_CLASE.CLASE="+idClase:"")+
				" ORDER BY UH_SEMANA.INICIO;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+","+idClase+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static void borrarUH(String idUH){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idUH+")");
		int ejecucion=-1;
		Vector<String> evaluables=null;
		Vector<String> evalEvaluables=null;
		if (idUH!=null) if (!idUH.isEmpty()) {
			String sql="SELECT EVALUABLES.ID" +
				" FROM EVALUABLES"+
				" WHERE EVALUABLES.UH_INICIO="+idUH+" OR EVALUABLES.UH_FIN="+idUH+";";
			evaluables=BBDD.consulta1ColumnaSQL(sql);
			sql="";
			if (evaluables!=null) if (evaluables.size()>0) {
				for (int j=0;j<evaluables.size();j++){
					evalEvaluables=BBDD.consultarEvaluacionesEvaluable(evaluables.get(j));
					if (evalEvaluables!=null) if (evalEvaluables.size()>0) {
						for (int i=0;i<evalEvaluables.size();i++){
							sql+="DELETE FROM EE_ESTUDIANTES WHERE E_EVALUABLE="+evalEvaluables.get(i)+";";
							sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE ID="+evalEvaluables.get(i)+";";
						}
					}
					sql+="DELETE FROM EVALUABLES WHERE ID="+evaluables.get(j)+";";
				}
			}
			sql+="DELETE FROM UH_SEMANA WHERE ID="+idUH+";";
			//System.out.println("sql:"+sql);
			ejecucion=BBDD.ejecutaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idUH+") evaluables ("+((evaluables!=null)?evaluables.size():"null")+") evalEvaluables ("+((evalEvaluables!=null)?evalEvaluables.size():"null")+") borrados "+ejecucion);
	}
	public static String insertarUHsiguiente(String idMateriaClase,String idDiaSemana){
		String id=null;
		Vector<Vector<String>> uhs=null;
		if (idMateriaClase!=null && idDiaSemana!=null) 
			if (!idMateriaClase.isEmpty() && !idDiaSemana.isEmpty()) {
				Calendar inicio=null;
				Calendar fin=null;
				String sql="SELECT HOUR(FIN),MINUTE(FIN) FROM UH_SEMANA WHERE UH_SEMANA.DIA_SEMANA="+idDiaSemana+
						" ORDER BY FIN ASC;";
				uhs=BBDD.consultaSQL(sql);
				if (uhs==null || uhs.size()==0) {
					inicio=Fecha.hora(Fecha.laboralPrimera, 0);
				} else {
					inicio=Fecha.hora(uhs.lastElement().get(0),uhs.lastElement().get(1));
				}
				fin=(Calendar)inicio.clone();
				fin.add(Calendar.HOUR_OF_DAY, +1);
				sql="INSERT INTO UH_SEMANA (MATERIACLASE,DIA_SEMANA,INICIO,FIN)"+
					" VALUES ("+idMateriaClase+","+idDiaSemana+",'"+Fecha.bbddSoloHora(inicio)+
					"','"+Fecha.bbddSoloHora(fin)+"');";
				id = BBDD.insertarIdSQL("UH_SEMANA",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+idDiaSemana+") uhs ("+uhs+") id del insertado = "+id);
		return id;
	}
	public static boolean consultarUHsolapada(String idUH,String idDiaSemana,String horaInicio,String minutoInicio,String horaFin,String minutoFin){
		boolean resultado=true;
		Vector<Vector<String>> tabla=null;
		if (idDiaSemana!=null && horaInicio!=null && minutoInicio!=null 
				&& horaFin!=null && minutoFin!=null) {
			if (!idDiaSemana.equals("") && !horaInicio.equals("") && !minutoInicio.equals("") 
					&& !horaFin.equals("") && !minutoFin.equals("")) {
				resultado=false;
				String inicio=Fecha.bbddSoloHora(Fecha.hora(horaInicio,minutoInicio));
				String fin=Fecha.bbddSoloHora(Fecha.hora(horaFin,minutoFin));
				String sql="SELECT UH_SEMANA.ID FROM UH_SEMANA" +
						" WHERE UH_SEMANA.ID<>"+idUH+" AND UH_SEMANA.DIA_SEMANA="+idDiaSemana+
						" AND ((INICIO<'"+inicio+"' AND FIN>'"+inicio+"')"+
						" OR (INICIO<'"+fin+"' AND FIN>'"+fin+"')" +
						" OR (INICIO='"+inicio+"' AND FIN='"+fin+"'));";
				tabla=BBDD.consultaSQL(sql);	
				//System.out.println("sql("+tabla.size()+"):"+sql);
				if (tabla!=null) if (tabla.size()>0) resultado=true;
			}
		}
		return resultado;
	}
	public static void modificarUH(String idUH,String idDiaSemana,String idMateriaClase,String horaInicio,String minutoInicio,String horaFin,String minutoFin){
		int ejecucion=-1;
		if (idDiaSemana!=null && idMateriaClase!=null && horaInicio!=null && minutoInicio!=null 
				&& horaFin!=null && minutoFin!=null) {
			if (!idDiaSemana.equals("") && !idMateriaClase.equals("") && !horaInicio.equals("") && !minutoInicio.equals("") 
					&& !horaFin.equals("") && !minutoFin.equals("")) {
				String sql="UPDATE UH_SEMANA SET DIA_SEMANA="+idDiaSemana+",MATERIACLASE="+idMateriaClase+
					",INICIO='"+Fecha.bbddSoloHora(Fecha.hora(horaInicio,minutoInicio))+"'"+
					",FIN='"+Fecha.bbddSoloHora(Fecha.hora(horaFin,minutoFin))+"'"+
					" WHERE UH_SEMANA.ID="+idUH+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idUH+","+idDiaSemana+","+idMateriaClase+","+horaInicio+","+minutoInicio+","+horaFin+","+minutoFin+") modificados "+ejecucion);		
	}
	public static Vector<Vector<String>> consultarUHMateriaClase(String idMateriaClase){
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null) if (!idMateriaClase.isEmpty()) {
			String sql="SELECT UH_SEMANA.ID,UH_SEMANA.DIA_SEMANA,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO)," +
					"HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN) FROM UH_SEMANA" +
					" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase+
					" ORDER BY UH_SEMANA.INICIO;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
		
	}
	public static String consultarIdUHoraria(Calendar fecha){
		Vector<Vector<String>> tabla=null;		// 8:30 a 9:30 es de 8:30 a 9:29
		String resultado=null;
		String sql="SELECT UH_SEMANA.ID" +
				" FROM UH_SEMANA" +
				" WHERE UH_SEMANA.DIA_SEMANA="+fecha.get(Calendar.DAY_OF_WEEK)+
				" AND UH_SEMANA.INICIO<='"+Fecha.bbddSoloHora(fecha)+"'"+
				" AND UH_SEMANA.FIN>'"+Fecha.bbddSoloHora(fecha)+"';";
		//System.out.println("consultarIdUHoraria: "+sql);
		tabla=BBDD.consultaSQL(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
						resultado=tabla.get(0).get(0);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(fecha)+") "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static Vector<Vector<String>> consultarUH(String diaSemana,String idUH,boolean ordenDiaSemana){
		Vector<Vector<String>> tabla=null;
		String sql="SELECT UH_SEMANA.ID,UH_SEMANA.MATERIACLASE,UH_SEMANA.DIA_SEMANA,HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO)" +
			",HOUR(UH_SEMANA.FIN),MINUTE(UH_SEMANA.FIN),MATERIAS.REFERENCIA,NIVELES.REFERENCIA,CLASES.REFERENCIA,TITULOS.REFERENCIA" +
			",ENSENANZAS.REFERENCIA FROM UH_SEMANA,MATERIA_CLASE,MATERIAS,CLASES,NIVELES,TITULOS,ENSENANZAS" +
			" WHERE UH_SEMANA.MATERIACLASE=MATERIA_CLASE.ID AND MATERIA_CLASE.MATERIA=MATERIAS.ID" +
			" AND MATERIA_CLASE.CLASE=CLASES.ID AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID" +
			" AND TITULOS.ENSENANZA=ENSENANZAS.ID"+
			((diaSemana!=null)?" AND UH_SEMANA.DIA_SEMANA="+diaSemana:"")+
			((idUH!=null)?" AND UH_SEMANA.ID="+idUH:"")+
			" ORDER BY "+((ordenDiaSemana)?"UH_SEMANA.DIA_SEMANA,":"")+"UH_SEMANA.INICIO,UH_SEMANA.FIN;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+diaSemana+","+idUH+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;	
	}
	public static void borrarEstudianteGrupoEvalEvaluable(String idEstudiante, String idEvalEvaluable){
		int ejecucion=-1;
		if (idEvalEvaluable!=null) {
			if (idEvalEvaluable!="") {
				String sql="DELETE FROM EE_ESTUDIANTES WHERE" +
						" E_EVALUABLE="+idEvalEvaluable+" AND ESTUDIANTE="+idEstudiante+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idEvalEvaluable+") borrados "+ejecucion);
	}
	public static void borrarEvaluable(String idEvaluable){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+")");
		int ejecucion=-1;
		Vector<String> registro=null;
		if (idEvaluable!=null) {
			String sql="";
			registro=BBDD.consultarEvaluacionesEvaluable(idEvaluable);
			if (registro!=null) if (registro.size()>0) {
				for (int i=0;i<registro.size();i++){
					sql+="DELETE FROM EE_ESTUDIANTES WHERE E_EVALUABLE="+registro.get(i)+";";
					sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE ID="+registro.get(i)+";";
				}
			}
			sql+="DELETE FROM EVALUABLES WHERE ID="+idEvaluable+";";
			ejecucion=BBDD.ejecutaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") evalEvaluables ("+((registro!=null)?registro.size():"null")+") borrados "+ejecucion);
	}
	public static void borrarEvalEvaluable(String idEvalEvaluable){
		int ejecucion=-1;
		if (idEvalEvaluable!=null) {
			if (idEvalEvaluable!="") {
				String sql="DELETE FROM EE_ESTUDIANTES WHERE E_EVALUABLE="+idEvalEvaluable+";";
				sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE ID="+idEvalEvaluable+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvalEvaluable+") borrados "+ejecucion);
	}
	public static String insertarEvalEstudianteEvaluable(String idEstudiante,String idEvaluable,String nota){
		int ejecucion=-1;
		String id=null;
		if (idEstudiante!=null && idEvaluable!=null) {
			if (idEstudiante!="" && idEvaluable!="") {
				String sql="INSERT INTO EVALUACIONES_EVALUABLES (EVALUABLE,NOTA,OBSERVACIONES,GRUPO)"+
				" VALUES ("+idEvaluable+","+String.valueOf(Numero.numeroDecimal(nota))+",'','');";
				//System.out.println("Insertando EvalEvaluable para evaluable ("+idEvaluable+") para estudiante ("+idEstudiante+")");
				id = BBDD.insertarIdSQL("EVALUACIONES_EVALUABLES",sql);
				if (id!=null) if (id!="") {
					sql="INSERT INTO EE_ESTUDIANTES (E_EVALUABLE,ESTUDIANTE)"+
					" VALUES ("+id+","+idEstudiante+");";
					ejecucion=BBDD.ejecutaSQL(sql);
					//System.out.println("Insertando EvalEvaluable ("+idEvalEvaluable+")");
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idEvaluable+","+nota+") idEvalEvaluable ("+id+") insertado "+ejecucion);
		return id;
	}
	public static void insertarEstudiantesMateriaClase(String idMateriaClase,Vector<String> idEstudiantes){
		int ejecucion=-1;
		if (idEstudiantes!=null && idMateriaClase!=null)
			if (idEstudiantes.size()>0 && !idMateriaClase.isEmpty()) {
				String sql="";
				for (int i=0;i<idEstudiantes.size();i++) {
					Vector<Vector<String>> estudiante=BBDD.consultarEstudiantesMateriaClase(idMateriaClase, idEstudiantes.get(i));
					if (estudiante.size()==0) 
						sql+="INSERT INTO ESTUDIANTE_MATERIACLASE" +
							" (MATERIACLASE,ESTUDIANTE) VALUES ("+idMateriaClase+","+idEstudiantes.get(i)+");";
				}
				if (!sql.isEmpty()) {
					if (PCuaderno.noti) ejecucion += BBDD.ejecutaSQL(sql);
					else BBDD.ejecutaSQL(sql);
				}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+idEstudiantes+") insertados = "+ejecucion);
	}
	public static void borrarEstudiantesMateriaClase(String idMateriaClase,Vector<String> idEstudiantes){
		int ejecucion=-1;
		Vector<Vector<String>> evalEvaluable=null;
		if (idEstudiantes!=null && idMateriaClase!=null)
			if (idEstudiantes.size()>0 && !idMateriaClase.isEmpty()) {
				String sql="";
				for (int j=0;j<idEstudiantes.size();j++) {
					// debe borrar la evaluaciones 
					evalEvaluable = BBDD.consultaSQL("SELECT EVALUACIONES_EVALUABLES.ID,EVALUACIONES_EVALUABLES.GRUPO" +
							" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
							" WHERE EVALUACIONES_EVALUABLES.ID=EE_ESTUDIANTES.E_EVALUABLE" +
							" AND EE_ESTUDIANTES.ESTUDIANTE="+idEstudiantes.get(j)+";");
					sql+="DELETE FROM EE_ESTUDIANTES WHERE EE_ESTUDIANTES.ESTUDIANTE="+idEstudiantes.get(j)+";";
					if (evalEvaluable!=null) if (evalEvaluable.size()>0) {
						for (int i=0;i<evalEvaluable.size();i++) {
							if (evalEvaluable.get(i).get(1).isEmpty()) 
								sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE EVALUACIONES_EVALUABLES.ID="+evalEvaluable.get(i).get(0)+";";
							else {
								Vector<Vector<String>> estudiantesGrupo=BBDD.consultarEstudiantesEvalEvaluable(evalEvaluable.get(i).get(0));
								if (estudiantesGrupo!=null) if (estudiantesGrupo.size()==1)
									sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE EVALUACIONES_EVALUABLES.ID="+evalEvaluable.get(i).get(0)+";";
							}
						}
					}
					sql+="DELETE FROM ESTUDIANTE_MATERIACLASE" +
							" WHERE MATERIACLASE="+idMateriaClase+" AND ESTUDIANTE="+idEstudiantes.get(j)+";";
				}
				if (PCuaderno.noti) ejecucion += BBDD.ejecutaSQL(sql);
				else BBDD.ejecutaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+idEstudiantes+") insertados = "+ejecucion);
	}
	public static void insertarEstudianteGrupoEvalEvaluable(String idEstudiante,String idEval){
		int ejecucion=-1;
		if (idEstudiante!=null && idEval!=null) {
			if (idEstudiante!="" && idEval!="") {
				String sql="INSERT INTO EE_ESTUDIANTES (E_EVALUABLE,ESTUDIANTE)"+
				" VALUES ("+idEval+","+idEstudiante+");";
				ejecucion=BBDD.ejecutaSQL(sql);
				//System.out.println("Insertando EvalEvaluable ("+idEvalEvaluable+")");
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idEval+") insertado "+ejecucion);
	}
	public static Vector<Vector<String>> consultarEstudiantesEvalEvaluable(String idEvalEvaluable){
		Vector<Vector<String>> tabla;
		String sql="SELECT EE_ESTUDIANTES.ESTUDIANTE,ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE" +
				" FROM EE_ESTUDIANTES,ESTUDIANTES" +
				" WHERE EE_ESTUDIANTES.ESTUDIANTE=ESTUDIANTES.ID" +
				" AND EE_ESTUDIANTES.E_EVALUABLE="+idEvalEvaluable+
				" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvalEvaluable+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static String consultarIdMateriaClaseEvaluable(String idEvaluable){
		String resultado=null;
		Vector<Vector<String>> tabla;
		String sql="SELECT UH_SEMANA.MATERIACLASE" +
				" FROM EVALUABLES,UH_SEMANA" +
				" WHERE EVALUABLES.UH_INICIO=UH_SEMANA.ID" +
				" AND EVALUABLES.ID="+idEvaluable+";";
		tabla=BBDD.consultaSQL(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
					if (tabla.get(0).size()>0) {
						resultado=tabla.get(0).get(0);
					}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static Vector<Vector<String>> consultarEstudiantesEvalSinGrupoOEvalEvaluable(String idEvaluable){
		Vector<Vector<String>> tabla=null;
		if (idEvaluable!=null) if (!idEvaluable.isEmpty()) {
			tabla=BBDD.consultarEstudiantesMateriaClase(BBDD.consultarIdMateriaClaseEvaluable(idEvaluable),null);
			Vector<String> estudiantesMateriaConGrupo;
			String sql="SELECT ESTUDIANTES.ID" +
			" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES,ESTUDIANTES" +
			" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable+
			" AND EVALUACIONES_EVALUABLES.GRUPO<>''" +
			" AND EE_ESTUDIANTES.E_EVALUABLE=EVALUACIONES_EVALUABLES.ID" +
			" AND EE_ESTUDIANTES.ESTUDIANTE=ESTUDIANTES.ID" +
			" ORDER BY ESTUDIANTES.ID;";
			estudiantesMateriaConGrupo=BBDD.consulta1ColumnaSQL(sql);
			if (estudiantesMateriaConGrupo!=null) if (estudiantesMateriaConGrupo.size()>0) {
				for (int i=0;i<estudiantesMateriaConGrupo.size();i++){
					for (int j=0;j<tabla.size();j++){
						if (estudiantesMateriaConGrupo.get(i).equals(tabla.get(j).get(0))) {
							tabla.removeElementAt(j);
							//System.out.println("Borrando id "+estudiantesMateriaConGrupo.get(i)+" ("+j+") tamaño "+tabla.size());
							j=tabla.size();
						}
					}
				}
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarIdEstudiantesEvalEvaluable(String idEvalEvaluable){
		Vector<Vector<String>> tabla=null;
		if (idEvalEvaluable!=null) if (!idEvalEvaluable.isEmpty()) {
			String sql="SELECT EE_ESTUDIANTES.ESTUDIANTE" +
					" FROM EE_ESTUDIANTES" +
					" WHERE EE_ESTUDIANTES.E_EVALUABLE="+idEvalEvaluable+";";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvalEvaluable+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static void modificarNotaEvalEvaluable(String nota,String idEvalEvaluable){
		int ejecucion=-1;
		if (idEvalEvaluable!=null) {
			if (!idEvalEvaluable.isEmpty()) {
				String sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.NOTA="+String.valueOf(Numero.numeroDecimal(nota))+
				" WHERE EVALUACIONES_EVALUABLES.ID="+idEvalEvaluable+";";
				//System.out.println("sql "+sql);
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+nota+","+idEvalEvaluable+") modificados "+ejecucion);
	}
	public static void modificarGrupoEvalEvaluable(String grupo,String idEvalEvaluable){
		int ejecucion=-1;
		if (idEvalEvaluable!=null && grupo!=null) {
			if (!idEvalEvaluable.isEmpty() && !grupo.isEmpty()) {
				String sql="UPDATE EVALUACIONES_EVALUABLES SET EVALUACIONES_EVALUABLES.GRUPO='"+grupo+"'"+
				" WHERE EVALUACIONES_EVALUABLES.ID="+idEvalEvaluable+";";
				//System.out.println("sql "+sql);
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+grupo+","+idEvalEvaluable+") modificados "+ejecucion);
	}
	public static String consultarIdEvalEstudianteEvaluable(String idEstudiante,String idEvaluable){
		Vector<Vector<String>> tabla;
		String resultado=null;
		String sql="SELECT DISTINCT EVALUACIONES_EVALUABLES.ID" +
				" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
				" WHERE EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante+
				" AND EE_ESTUDIANTES.E_EVALUABLE=EVALUACIONES_EVALUABLES.ID" +
				" AND EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable+";";
		tabla=BBDD.consultaSQL(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
					if (tabla.get(0).size()>0) {
						resultado=tabla.get(0).get(0);
						//System.out.println("(estudiante="+idEstudiante+",evaluable="+idEvaluable+") idEval = "+resultado);
					}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static String insertarTipoEvaluable(String idGrupoMateria, String referencia, String descripcion, Integer peso, String minimo){
		String id=null;
		if (referencia!=null && descripcion!=null && peso!=null) {
			if (idGrupoMateria!=null) if (idGrupoMateria.equals(BBDD.idTodos)) idGrupoMateria=null;
			String sql="INSERT INTO TIPOS_EVALUABLE (MATERIA,REFERENCIA,DESCRIPCION,PESO,MINIMO)"+
			" VALUES ("+((idGrupoMateria==null)?"NULL":idGrupoMateria)+",'"+referencia+
			"','"+descripcion+"',"+peso+","+String.valueOf(Numero.numeroDecimal(minimo))+");";
			id = BBDD.insertarIdSQL("EVENTOS",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idGrupoMateria+","+referencia+","+descripcion+","+peso+","+minimo+") id del insertado "+id);
		return id;
	}
	public static void borrarTipoEvaluable(String idTipo){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTipo+")");
		int ejecucion=-1;
		Vector<String> evaluables=null;
		Vector<String> evalEvaluables=null;
		if (idTipo!=null) if (!idTipo.isEmpty()) {
			String sql="SELECT EVALUABLES.ID" +
				" FROM EVALUABLES"+
				" WHERE EVALUABLES.TIPO="+idTipo+";";
			evaluables=BBDD.consulta1ColumnaSQL(sql);
			sql="";
			if (evaluables!=null) if (evaluables.size()>0) {
				for (int j=0;j<evaluables.size();j++){
					evalEvaluables=BBDD.consultarEvaluacionesEvaluable(evaluables.get(j));
					if (evalEvaluables!=null) if (evalEvaluables.size()>0) {
						for (int i=0;i<evalEvaluables.size();i++){
							sql+="DELETE FROM EE_ESTUDIANTES WHERE E_EVALUABLE="+evalEvaluables.get(i)+";";
							sql+="DELETE FROM EVALUACIONES_EVALUABLES WHERE ID="+evalEvaluables.get(i)+";";
						}
					}
					sql+="DELETE FROM EVALUABLES WHERE ID="+evaluables.get(j)+";";
				}
			}
			sql+="DELETE FROM TIPOS_EVALUABLE WHERE ID="+idTipo+";";
			ejecucion=BBDD.ejecutaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idTipo+") evaluables ("+((evaluables!=null)?evaluables.size():"null")+") evalEvaluables ("+((evalEvaluables!=null)?evalEvaluables.size():"null")+") borrados "+ejecucion);
	}
	public static Vector<Vector<String>> consultarTiposEvaluableMateriaClaseParcial(String idMateriaClase,Integer parcial){
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null && parcial!=null) if (!idMateriaClase.isEmpty() && parcial>0) {
			Calendar ip=((parcial>1)?Parcial.getFechaInicioParcial(parcial):Fecha.inicioCurso);
			Calendar fp=((parcial<Parcial.numeroParciales())?Fecha.diaLaboralOAnterior(Parcial.getFechaInicioParcial(parcial+1),true):Fecha.finCurso);
			String sql="SELECT DISTINCT TIPOS_EVALUABLE.ID, TIPOS_EVALUABLE.REFERENCIA," +
					" TIPOS_EVALUABLE.DESCRIPCION, TIPOS_EVALUABLE.PESO, TIPOS_EVALUABLE.MINIMO" +
					" FROM TIPOS_EVALUABLE,EVALUABLES,UH_SEMANA" +
					" WHERE UH_SEMANA.MATERIACLASE="+idMateriaClase+
					" AND EVALUABLES.UH_INICIO=UH_SEMANA.ID" +
					" AND EVALUABLES.DIA_FIN>='"+Fecha.bbddSoloDia(ip)+"'"+
					" AND EVALUABLES.DIA_FIN<='"+Fecha.bbddSoloDia(fp)+"'"+
					" AND EVALUABLES.TIPO=TIPOS_EVALUABLE.ID" +
					" ORDER BY TIPOS_EVALUABLE.PESO ASC;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+parcial+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarTiposEvaluablesParaMateria(String idMateria){
		Vector<Vector<String>> tabla=null;  // Si idGrupoMateria es nulo obtiene los tipos que afectan a todas las materias
		String sql="SELECT TIPOS_EVALUABLE.ID, TIPOS_EVALUABLE.REFERENCIA," +
				" TIPOS_EVALUABLE.DESCRIPCION, TIPOS_EVALUABLE.PESO, TIPOS_EVALUABLE.MINIMO" +
				" FROM TIPOS_EVALUABLE" +
				" WHERE TIPOS_EVALUABLE.MATERIA"+((idMateria==null)?" IS NULL":"="+idMateria)+
				" ORDER BY TIPOS_EVALUABLE.REFERENCIA;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static boolean consultarTipoEvaluableVacio(String idMateria){
		boolean resultado=true;
		Vector<Vector<String>> tabla=null;
		String sql="SELECT TIPOS_EVALUABLE.ID FROM TIPOS_EVALUABLE" +
				" WHERE TIPOS_EVALUABLE.MATERIA IS NULL"+
				((idMateria!=null)?" OR TIPOS_EVALUABLE.MATERIA="+idMateria:"")+
				" LIMIT 1;";
		tabla=BBDD.consultaSQL(sql);	
		if (tabla!=null) if (tabla.size()>0) resultado=false;
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static Vector<Vector<String>> consultarTiposEvaluables(String id){
		Vector<Vector<String>> tabla; // id = nulo para consultarlos todos
		String sql="SELECT TIPOS_EVALUABLE.ID, TIPOS_EVALUABLE.REFERENCIA," +
				" TIPOS_EVALUABLE.DESCRIPCION, TIPOS_EVALUABLE.PESO" +
				" TIPOS_EVALUABLE.MATERIA, TIPOS_EVALUABLE.MINIMO" +
				" FROM TIPOS_EVALUABLE" +
				((id!=null)?" WHERE TIPOS_EVALUABLE.ID="+id:"")+
				" ORDER BY TIPOS_EVALUABLE.REFERENCIA;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+id+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<String> consultarEvaluacionesEvaluable(String idEvaluable){
		Vector<String> registro=null;
		if (idEvaluable!=null) if (!idEvaluable.isEmpty()) {
			String sql="SELECT EVALUACIONES_EVALUABLES.ID" +
					" FROM EVALUACIONES_EVALUABLES" +
					" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable +";";
			registro=BBDD.consulta1ColumnaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((registro!=null)?registro.size():"null")+" registros");
		return registro;
	}
	public static Vector<Vector<String>> consultarEvaluacionesEvaluableGrupos(String idEvaluable){
		Vector<Vector<String>> tabla=null;
		if (idEvaluable!=null) if (!idEvaluable.isEmpty()) {
			String sql="SELECT EVALUACIONES_EVALUABLES.ID, EVALUACIONES_EVALUABLES.EVALUABLE," +
					" EVALUACIONES_EVALUABLES.NOTA,EVALUACIONES_EVALUABLES.OBSERVACIONES, EVALUACIONES_EVALUABLES.GRUPO" +
					" FROM EVALUACIONES_EVALUABLES" +
					" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable +
					" AND EVALUACIONES_EVALUABLES.GRUPO<>''" +
					" ORDER BY EVALUACIONES_EVALUABLES.GRUPO;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static String consultarEvalNota(String idEvaluable,String idEstudiante){
		Vector<Vector<String>> tabla=null;
		String resultado=null;
		if (idEvaluable!=null && idEstudiante!=null) if (!idEvaluable.isEmpty() && !idEstudiante.isEmpty()) {
			String sql="SELECT EVALUACIONES_EVALUABLES.NOTA" +
					" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
					" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable +
					" AND EVALUACIONES_EVALUABLES.ID=EE_ESTUDIANTES.E_EVALUABLE" +
					" AND EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante +
					" ORDER BY EVALUACIONES_EVALUABLES.GRUPO;";
			tabla=BBDD.consultaSQL(sql);
			if (tabla!=null)
				if (tabla.size()>0)
					if (tabla.get(0)!=null)
							resultado=tabla.get(0).get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static String consultarEvalObservaciones(String idEvaluable,String idEstudiante){
		Vector<Vector<String>> tabla=null;
		String resultado=null;
		if (idEvaluable!=null && idEstudiante!=null) if (!idEvaluable.isEmpty() && !idEstudiante.isEmpty()) {
			String sql="SELECT EVALUACIONES_EVALUABLES.OBSERVACIONES" +
					" FROM EVALUACIONES_EVALUABLES,EE_ESTUDIANTES" +
					" WHERE EVALUACIONES_EVALUABLES.EVALUABLE="+idEvaluable +
					" AND EVALUACIONES_EVALUABLES.ID=EE_ESTUDIANTES.E_EVALUABLE" +
					" AND EE_ESTUDIANTES.ESTUDIANTE="+idEstudiante +
					" ORDER BY EVALUACIONES_EVALUABLES.GRUPO;";
			tabla=BBDD.consultaSQL(sql);
			if (tabla!=null)
				if (tabla.size()>0)
					if (tabla.get(0)!=null)
							resultado=tabla.get(0).get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static Vector<Vector<String>> consultarEval(String idEval){
		Vector<Vector<String>> tabla=null;
		if (idEval!=null) if (!idEval.isEmpty()) {
			String sql="SELECT ID,EVALUABLE,NOTA,OBSERVACIONES,GRUPO" +
					" FROM EVALUACIONES_EVALUABLES" +
					" WHERE EVALUACIONES_EVALUABLES.ID="+idEval+";";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEval+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEvalEvaluablesPorEstudianteEnMateriaClaseParcial(String idEstudiante,String idMateriaClase,Integer parcial){
		Vector<Vector<String>> tabla=null;
		if (idEstudiante!=null && idMateriaClase!=null && parcial!=null) if (!idEstudiante.isEmpty() && !idMateriaClase.isEmpty() && parcial>0) {
			Calendar ip=((parcial>1)?Parcial.getFechaInicioParcial(parcial):Fecha.inicioCurso);
			Calendar fp=((parcial<Parcial.numeroParciales())?Fecha.diaLaboralOAnterior(Parcial.getFechaInicioParcial(parcial+1),true):Fecha.finCurso);
			String sql="SELECT EVALUACIONES_EVALUABLES.ID, EVALUACIONES_EVALUABLES.EVALUABLE," +
					" EVALUACIONES_EVALUABLES.NOTA,EVALUACIONES_EVALUABLES.OBSERVACIONES, EVALUACIONES_EVALUABLES.GRUPO" +
					" FROM UH_SEMANA,EVALUABLES,EVALUACIONES_EVALUABLES,EE_ESTUDIANTES,ESTUDIANTES" +
					" WHERE ESTUDIANTES.ID="+idEstudiante+
					" AND EE_ESTUDIANTES.ESTUDIANTE=ESTUDIANTES.ID" +
					" AND EE_ESTUDIANTES.E_EVALUABLE=EVALUACIONES_EVALUABLES.ID" +
					" AND EVALUACIONES_EVALUABLES.EVALUABLE=EVALUABLES.ID" +
					" AND EVALUABLES.UH_INICIO=UH_SEMANA.ID" +
					" AND UH_SEMANA.MATERIACLASE="+idMateriaClase+
					" AND EVALUABLES.DIA_FIN>='"+Fecha.bbddSoloDia(ip)+"'"+
					" AND EVALUABLES.DIA_FIN<='"+Fecha.bbddSoloDia(fp)+"'"+
					" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN ASC;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEstudiante+","+idMateriaClase+","+parcial+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEvaluable(String idEvaluable){
		Vector<Vector<String>> tabla=null;
		if (idEvaluable!=null) if (!idEvaluable.isEmpty()) {
			String sql="SELECT EVALUABLES.ID" +
					",YEAR(EVALUABLES.DIA_INICIO),MONTH(EVALUABLES.DIA_INICIO),DAYOFMONTH(EVALUABLES.DIA_INICIO),EVALUABLES.UH_INICIO"+
					",YEAR(EVALUABLES.DIA_FIN),MONTH(EVALUABLES.DIA_FIN),DAYOFMONTH(EVALUABLES.DIA_FIN),EVALUABLES.UH_FIN"+
			/*9*/	",EVALUABLES.REFERENCIA, EVALUABLES.ENUNCIADO, EVALUABLES.PESO, TIPOS_EVALUABLE.ID"+
					",TIPOS_EVALUABLE.REFERENCIA, TIPOS_EVALUABLE.DESCRIPCION"+
					" FROM EVALUABLES,TIPOS_EVALUABLE,UH_SEMANA"+
					" WHERE EVALUABLES.TIPO=TIPOS_EVALUABLE.ID"+
					" AND UH_SEMANA.ID=EVALUABLES.UH_INICIO"+
					" AND EVALUABLES.ID="+idEvaluable+
					" ORDER BY EVALUABLES.DIA_FIN ASC;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idEvaluable+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static boolean consultarEvaluableConUHvacio(String idUH){
		boolean resultado=true;
		Vector<Vector<String>> tabla=null;
		if (idUH!=null) if (!idUH.isEmpty()) { // UH_FIN Manda sobre UH_INICIO para MateriaClase
			String sql="SELECT ID FROM EVALUABLES" +
					" WHERE EVALUABLES.UH_FIN="+idUH+
					" LIMIT 1;";
			tabla=BBDD.consultaSQL(sql);	
			if (tabla!=null) if (tabla.size()>0) resultado=false;
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idUH+") "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static Vector<Vector<String>> consultarEvaluablesMateriaClaseParcial(String idMateriaClase,Integer parcial){
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null && parcial!=null) if (!idMateriaClase.isEmpty() && parcial>0) {
			Calendar ip=((parcial>1)?Parcial.getFechaInicioParcial(parcial):Fecha.inicioCurso);
			Calendar fp=((parcial<Parcial.numeroParciales())?Fecha.diaLaboralOAnterior(Parcial.getFechaInicioParcial(parcial+1),true):Fecha.finCurso);
			String sql="SELECT EVALUABLES.ID" +
					",YEAR(EVALUABLES.DIA_FIN), MONTH(EVALUABLES.DIA_FIN), DAYOFMONTH(EVALUABLES.DIA_FIN)" +
					",HOUR(UH_SEMANA.FIN), MINUTE(UH_SEMANA.FIN), EVALUABLES.REFERENCIA" +
					",EVALUABLES.ENUNCIADO, EVALUABLES.PESO, TIPOS_EVALUABLE.ID" +
					",TIPOS_EVALUABLE.REFERENCIA, TIPOS_EVALUABLE.DESCRIPCION" +
					" FROM EVALUABLES,TIPOS_EVALUABLE,UH_SEMANA" +
					" WHERE EVALUABLES.TIPO=TIPOS_EVALUABLE.ID" +
					" AND UH_SEMANA.ID=EVALUABLES.UH_FIN" + // UH_FIN Manda sobre UH_INICIO para MateriaClase
					" AND EVALUABLES.DIA_FIN>='"+Fecha.bbddSoloDia(ip)+"'"+
					" AND EVALUABLES.DIA_FIN<='"+Fecha.bbddSoloDia(fp)+"'"+
					" AND UH_SEMANA.MATERIACLASE="+idMateriaClase +
					" ORDER BY EVALUABLES.DIA_FIN,UH_SEMANA.FIN ASC;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+parcial+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEstudiantesMateriaClase(String idMateriaClase,String idEstudiante){
		Vector<Vector<String>> tabla = new Vector<Vector<String>>();
		String sql="SELECT DISTINCT ESTUDIANTES.ID,APELLIDOS,NOMBRE,DATOS" +
				" FROM ESTUDIANTES, ESTUDIANTE_MATERIACLASE" +
				" WHERE ESTUDIANTES.ID=ESTUDIANTE_MATERIACLASE.ESTUDIANTE" +
				((idMateriaClase!=null)?" AND ESTUDIANTE_MATERIACLASE.MATERIACLASE="+idMateriaClase:"")+
				((idEstudiante!=null)?" AND ESTUDIANTES.ID="+idEstudiante:"")+
				" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+","+idEstudiante+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEstudiantesClase(String idClase,String idEstudiante){
		Vector<Vector<String>> tabla = new Vector<Vector<String>>();
		String sql="SELECT ESTUDIANTES.ID,APELLIDOS,NOMBRE,DATOS" +
				" FROM ESTUDIANTES" +
				((idClase!=null || idEstudiante!=null)?" WHERE":"")+
				((idClase!=null)?" ESTUDIANTES.CLASE="+idClase:"")+
				((idClase!=null && idEstudiante!=null)?" AND":"")+
				((idEstudiante!=null)?" ESTUDIANTES.ID="+idEstudiante:"")+
				" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+","+idEstudiante+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarEstudiantesMateria(String idMateria,String idEstudiante){
		Vector<Vector<String>> tabla = new Vector<Vector<String>>();
		String sql="SELECT DISTINCT ESTUDIANTES.ID,APELLIDOS,NOMBRE,DATOS" +
				" FROM ESTUDIANTES, ESTUDIANTE_MATERIACLASE, MATERIA_CLASE" +
				" WHERE ESTUDIANTES.ID=ESTUDIANTE_MATERIACLASE.ESTUDIANTE" +
				((idMateria!=null)?" AND ESTUDIANTE_MATERIACLASE.MATERIACLASE=MATERIA_CLASE.ID" +
				" AND MATERIA_CLASE.MATERIA="+idMateria:"")+
				((idEstudiante!=null)?" AND ESTUDIANTES.ID="+idEstudiante:"")+
				" ORDER BY ESTUDIANTES.APELLIDOS,ESTUDIANTES.NOMBRE;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+","+idEstudiante+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarMateria(String idMateria){
		Vector<Vector<String>> tabla;
		String sql="SELECT MATERIAS.ID,MATERIAS.REFERENCIA,MATERIAS.DESCRIPCION,MATERIAS.NIVEL"+
				",NIVELES.REFERENCIA,TITULOS.REFERENCIA,ENSENANZAS.REFERENCIA FROM MATERIAS,NIVELES,TITULOS,ENSENANZAS"+
				" WHERE MATERIAS.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID AND TITULOS.ENSENANZA=ENSENANZAS.ID"+
				((idMateria!=null)?" AND MATERIAS.ID="+idMateria:"")+
				" ORDER BY ENSENANZAS.REFERENCIA,TITULOS.REFERENCIA,NIVELES.REFERENCIA,MATERIAS.REFERENCIA;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static String insertarMateria(String idNivel,String referencia,String titulo){
		String id=null;
		int ejecucion=0;
		Vector<Vector<String>> clases=null;
		if (idNivel!=null && referencia!=null && titulo!=null) 
			if (!idNivel.isEmpty() && !referencia.isEmpty()) {
				String sql="INSERT INTO MATERIAS (NIVEL,REFERENCIA,DESCRIPCION)" +
						" VALUES ("+idNivel+",'"+referencia+"','"+titulo+"');";
				id = BBDD.insertarIdSQL("CLASES",sql);
				sql="";
				clases = BBDD.consultarClase(idNivel, null);
				if (clases!=null) if (clases.size()>0) {
					for (int i=0;i<clases.size();i++) 
						sql+="INSERT INTO MATERIA_CLASE (MATERIA,CLASE) VALUES ("+id+","+clases.get(i).get(0)+");";
				}
				ejecucion=BBDD.ejecutaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+","+referencia+","+titulo+") clases ("+((clases!=null)?clases.size():"null")+") id del insertado = "+id+" insertados "+ejecucion);
		return id;
	}
	public static Vector<Vector<String>> consultarClase(String idNivel,String idClase){
		Vector<Vector<String>> tabla;
		String sql="SELECT CLASES.ID,NIVELES.REFERENCIA,CLASES.REFERENCIA,TITULOS.REFERENCIA" +
				",PROFESORES.ID,PROFESORES.NOMBRE,PROFESORES.APELLIDOS" +
				" FROM CLASES,NIVELES,TITULOS,PROFESORES" +
				" WHERE CLASES.TUTOR=PROFESORES.ID AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID"+
				((idNivel!=null)?" AND NIVELES.ID="+idNivel:"")+
				((idClase!=null)?" AND CLASES.ID="+idClase:"")+
				" ORDER BY TITULOS.REFERENCIA,NIVELES.REFERENCIA,CLASES.REFERENCIA;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNivel+","+idNivel+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarClasesParaMateria(String idMateria){
		Vector<Vector<String>> tabla=null;
		if (idMateria!=null) {
			if (!idMateria.isEmpty()) {
			String sql="SELECT CLASES.ID,NIVELES.REFERENCIA,CLASES.REFERENCIA,TITULOS.REFERENCIA" +
					",PROFESORES.ID,PROFESORES.NOMBRE,PROFESORES.APELLIDOS" +
					" FROM MATERIAS,MATERIA_CLASE,CLASES,PROFESORES,NIVELES,TITULOS" +
					" WHERE MATERIA_CLASE.MATERIA=MATERIAS.ID AND MATERIA_CLASE.CLASE=CLASES.ID" +
					" AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID" +
					" AND CLASES.TUTOR=PROFESORES.ID AND MATERIAS.ID="+idMateria+
					" ORDER BY MATERIAS.REFERENCIA;";
			tabla=BBDD.consultaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateria+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarMateriaClase(){
		Vector<Vector<String>> tabla;
		String sql="SELECT MATERIA_CLASE.ID,NIVELES.REFERENCIA,CLASES.REFERENCIA,TITULOS.REFERENCIA,MATERIAS.REFERENCIA," +
				" MATERIAS.DESCRIPCION FROM MATERIAS,MATERIA_CLASE,CLASES,NIVELES,TITULOS" +
				" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND MATERIA_CLASE.CLASE=CLASES.ID" +
				" AND CLASES.NIVEL=NIVELES.ID AND NIVELES.TITULO=TITULOS.ID"+
				" ORDER BY TITULOS.REFERENCIA,NIVELES.REFERENCIA,CLASES.REFERENCIA,MATERIAS.REFERENCIA;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Integer> consultarDiasSemanaMateriaClase(String idMateriaClase){
		Vector<Integer> dias = new Vector<Integer>();
		Vector<Vector<String>> tabla;
		String sql="SELECT DISTINCT DIA_SEMANA FROM UH_SEMANA WHERE MATERIACLASE="+idMateriaClase+";";
		tabla=BBDD.consultaSQL(sql);
		if (tabla!=null) if (tabla.size()>0){
			for (int i=0;i<tabla.size();i++) {
				dias.add(Integer.parseInt(tabla.get(i).get(0)));
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idMateriaClase+") "+((dias!=null)?dias.size():"null")+" dias");
		return dias;
	}
	public static Calendar consultarHoraSiguienteParaFechaMateriaClase(Calendar fecha,String idMateriaClase){
		Calendar resultado = null;
		Vector<Vector<String>> tabla=null;
		if (fecha!=null) {
			String sql="SELECT HOUR(UH_SEMANA.INICIO),MINUTE(UH_SEMANA.INICIO) FROM UH_SEMANA" +
			" WHERE UH_SEMANA.DIA_SEMANA="+ fecha.get(Calendar.DAY_OF_WEEK) +
			" AND UH_SEMANA.INICIO>'"+Fecha.bbddSoloHora(fecha)+"'"+
			((idMateriaClase==null)?"":" AND UH_SEMANA.MATERIACLASE="+idMateriaClase) +
			" ORDER BY UH_SEMANA.INICIO;";
			tabla=BBDD.consultaSQL(sql);
			if (tabla!=null)
				if (tabla.size()>0)
					if (tabla.get(0)!=null)
						if (tabla.get(0).size()>0) {
							resultado = (Calendar) fecha.clone();
							resultado.set(Calendar.HOUR_OF_DAY, Integer.parseInt(tabla.get(0).get(0)));
							resultado.set(Calendar.MINUTE, Integer.parseInt(tabla.get(0).get(1)));
						}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(fecha)+","+idMateriaClase+") "+((tabla!=null)?tabla.size():"null")+" registros => "+((resultado==null)?"null":Fecha.formaFechaDiaHora(resultado)));
		return resultado;
	}
	public static Vector<String> consultarIdMateriaClase(String idMateria,String idClase){
		String sql=null;
		Vector<String> registro=null;
		Vector<Vector<String>> tabla=null;
		if (idMateria!=null && idClase!=null) {
			sql="SELECT MATERIA_CLASE.ID,MATERIA_CLASE.MATERIA,MATERIA_CLASE.CLASE" +
					" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
					" FROM MATERIA_CLASE,MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
					" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND NIVELES.ID=MATERIAS.NIVEL" +
					" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
					" AND MATERIA_CLASE.MATERIA="+idMateria+
					" AND MATERIA_CLASE.CLASE="+idClase+";";
		} else if (idMateria!=null && idClase==null) {
			sql="SELECT NULL,MATERIAS.ID,NULL" +
					" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
					" FROM MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
					" WHERE NIVELES.ID=MATERIAS.NIVEL" +
					" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
					((idMateria!=null)?" AND MATERIAS.ID="+idMateria:"")+";";
		} else if (idMateria==null && idClase!=null) {
			sql="SELECT NULL,NULL,CLASES.ID" +
					" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
					" FROM CLASES,NIVELES,TITULOS,ENSENANZAS" +
					" WHERE NIVELES.ID=CLASES.NIVEL" +
					" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
					((idClase!=null)?" AND CLASES.ID="+idClase:"")+";";
		}
		if (sql!=null) {
			tabla=BBDD.consultaSQL(sql);
			if (tabla!=null) if (tabla.size()>0) registro=tabla.get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" registros => "+((registro!=null)?registro.size():"null")+" datos");
		return registro;
	}
	public static Vector<String> consultarIdMateriaClase(String idMateriaClase){
		Vector<String> registro=null;
		Vector<Vector<String>> tabla=null;
		if (idMateriaClase!=null) {
			String sql="SELECT MATERIA_CLASE.ID,MATERIA_CLASE.MATERIA,MATERIA_CLASE.CLASE" +
					" ,NIVELES.ID,TITULOS.ID,ENSENANZAS.ID"+
					" FROM MATERIA_CLASE,MATERIAS,NIVELES,TITULOS,ENSENANZAS" +
					" WHERE MATERIAS.ID=MATERIA_CLASE.MATERIA AND NIVELES.ID=MATERIAS.NIVEL" +
					" AND TITULOS.ID=NIVELES.TITULO AND ENSENANZAS.ID=TITULOS.ENSENANZA"+
					" AND MATERIA_CLASE.ID="+idMateriaClase+";";
			tabla=BBDD.consultaSQL(sql);
			if (tabla!=null) if (tabla.size()>0) registro=tabla.get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" registros => "+((registro!=null)?registro.size():"null")+" datos");
		return registro;
	}
	public static String consultarIdMateriaClasePrimera(){
		String resultado=null;
		Vector<Vector<String>> tabla;
		String sql="SELECT ID FROM MATERIA_CLASE LIMIT 1;";
		tabla=BBDD.consultaSQL(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
					if (tabla.get(0).size()>0) {
						resultado=tabla.get(0).get(0);
					}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static String consultarIdMateriaClaseParaFechaHora(Calendar c){
		String resultado=null;
		Vector<Vector<String>> tabla;
		String sql="SELECT MATERIACLASE FROM UH_SEMANA" +
		" WHERE UH_SEMANA.DIA_SEMANA="+ c.get(Calendar.DAY_OF_WEEK) + " AND"+
		" UH_SEMANA.INICIO<='"+String.format("%02d",c.get(Calendar.HOUR_OF_DAY))+":"+String.format("%02d",c.get(Calendar.MINUTE))+":00' AND"+
		" UH_SEMANA.FIN>'"+String.format("%02d",c.get(Calendar.HOUR_OF_DAY))+":"+String.format("%02d",c.get(Calendar.MINUTE))+":00';";
		tabla=BBDD.consultaSQL(sql);
		if (tabla!=null)
			if (tabla.size()>0)
				if (tabla.get(0)!=null)
					if (tabla.get(0).size()>0) {
						resultado=tabla.get(0).get(0);
					}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(c)+") "+((tabla!=null)?tabla.size():"null")+" registros => "+resultado);
		return resultado;
	}
	public static Vector<String> consultarDiaNoLectivo(String idNoLectivo){
		Vector<String> dia=null;
		Vector<Vector<String>> tabla=null;
		if (idNoLectivo!=null) if (!idNoLectivo.isEmpty()) {
			String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA), MONTH(FECHA), YEAR(FECHA), DESCRIPCION FROM DIASNOLECTIVOS WHERE ID='"+idNoLectivo+"';";
			tabla=BBDD.consultaSQL(sql);
			if (tabla!=null) if (tabla.size()>0) if (tabla.get(0)!=null) if (tabla.get(0).size()>0) dia=tabla.get(0);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNoLectivo+") "+((dia!=null)?Fecha.formaFecha(dia.get(2), dia.get(3), dia.get(4))+": "+dia.get(5):"null"));
		return dia;
	}
	public static Vector<Vector<String>> consultarDiaNoLectivo(Calendar fecha){
		Vector<Vector<String>> tabla=null;
		if (fecha!=null) {
			String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA), MONTH(FECHA), YEAR(FECHA),DESCRIPCION FROM DIASNOLECTIVOS "+
				" WHERE DAYOFMONTH(FECHA)="+fecha.get(Calendar.DAY_OF_MONTH)+" AND"+
				" MONTH(FECHA)="+(fecha.get(Calendar.MONTH)+1)+" AND"+
				" YEAR(FECHA)="+fecha.get(Calendar.YEAR)+
				" ORDER BY CLASE;";
			tabla=BBDD.consultaSQL(sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(fecha)+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static void modificarDiaNoLectivoFecha(String idNoLectivo, Calendar fecha){
		int ejecucion=-1;
		if (idNoLectivo!=null && fecha!=null) {
			if (!idNoLectivo.equals("")) {
				String sql="UPDATE DIASNOLECTIVOS SET DIASNOLECTIVOS.FECHA='"+Fecha.bbddSoloDia(fecha)+"'"+
				" WHERE DIASNOLECTIVOS.ID="+idNoLectivo+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNoLectivo+","+Fecha.formaFechaDiaHora(fecha)+") modificados "+ejecucion);
	}
	public static void modificarDiaNoLectivoClase(String idNoLectivo, String clase){
		int ejecucion=-1;
		if (idNoLectivo!=null) {
			if (!idNoLectivo.isEmpty()) {
				String sql="UPDATE DIASNOLECTIVOS SET DIASNOLECTIVOS.CLASE="+((clase==null)?"NULL":clase)+
				" WHERE DIASNOLECTIVOS.ID="+idNoLectivo+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNoLectivo+","+clase+") modificados "+ejecucion);
	}
	public static void borrarDiaNoLectivo(String idNoLectivo){
		int ejecucion=-1;
		if (idNoLectivo!=null) if (!idNoLectivo.isEmpty()) {
				String sql="DELETE FROM DIASNOLECTIVOS WHERE ID="+idNoLectivo+";";
				ejecucion=BBDD.ejecutaSQL(sql);
			}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idNoLectivo+") borrados "+ejecucion);
	}
	public static String insertarDiaNoLectivo(Calendar fecha,String idClase,String descripcion){
		String id=null;
		if (fecha!=null && descripcion!=null) {
			String sql="INSERT INTO DIASNOLECTIVOS (CLASE,FECHA,DESCRIPCION)"+
			" VALUES ("+((idClase==null)?"NULL":idClase)+",'"+Fecha.bbddSoloDia(fecha)+"','"+descripcion+"');";
			id = BBDD.insertarIdSQL("DIASNOLECTIVOS",sql);
		}
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+Fecha.formaFechaDiaHora(fecha)+","+idClase+","+descripcion+") id del insertado "+id);
		return id;
	}
	public static Vector<Vector<String>> consultarDiasNoLectivosDiaCompletoYClase(String idClase){
		Vector<Vector<String>> tabla; 
		String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA)," +
				" MONTH(FECHA), YEAR(FECHA)," +
				" DESCRIPCION FROM DIASNOLECTIVOS" +
				" WHERE CLASE IS NULL"+((idClase!=null)?" OR CLASE="+idClase:"")+
				" ORDER BY FECHA ASC;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarDiasNoLectivosClase(String idClase){
		Vector<Vector<String>> tabla; 
		String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA)," +
				" MONTH(FECHA), YEAR(FECHA)," +
				" DESCRIPCION FROM DIASNOLECTIVOS" +
				" WHERE "+((idClase!=null)?"CLASE="+idClase:"CLASE IS NOT NULL")+
				" ORDER BY FECHA ASC;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+idClase+") "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static Vector<Vector<String>> consultarDiasNoLectivos(){
		Vector<Vector<String>> tabla; 
		String sql="SELECT ID,CLASE,DAYOFMONTH(FECHA)," +
				" MONTH(FECHA), YEAR(FECHA)," +
				" DESCRIPCION FROM DIASNOLECTIVOS" +
				" ORDER BY FECHA ASC;";
		tabla=BBDD.consultaSQL(sql);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("() "+((tabla!=null)?tabla.size():"null")+" registros");
		return tabla;
	}
	public static void imprimir(Vector<Vector<String>> datos){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((datos!=null)?datos.size():null)+")");
		String s=null;
		if (datos!=null) {
			if (datos.size()>0) {
				for (int i=0;i<datos.size();i++){
					s=i+":[ ";
					for (int j=0;j<datos.get(i).size();j++){
						s=s+"("+j+")=\""+datos.get(i).get(j)+((j<datos.get(i).size()-1)?"\" | ":" ");
					}
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(s+"]");
				}
			} else if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Vacio");
		} else if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Nulo");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("fin");
	}
	public static void imprimirRegistro(Vector<String> datos){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("("+((datos!=null)?datos.size():null)+")");
		String s=null;
		if (datos!=null) {
			if (datos.size()>0) {
				s="[ ";
				for (int j=0;j<datos.size();j++)
					s=s+"("+j+")=\""+datos.get(j)+((j<datos.size()-1)?"\" | ":" ");
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(s+"]");
			} else if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Vacio");
		} else if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Nulo");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("fin");
	}
	public static Vector<Vector<String>> consultaSQL(String sql){
		Connection tConexion = null;Statement sentencia = null;ResultSet resultado = null;
		Vector<Vector<String>> salida = new Vector<Vector<String>>();
		Vector<String> fila=null;
		try {tConexion = DriverManager.getConnection(BBDD.cadenaConexion);
		} catch (SQLException e) {
			System.out.println("BBDD.consultaSQL - No pudo conectar con la base de datos.");
			e.printStackTrace();
			}
		//System.out.println("ConsultaSQL = "+sql);
		try {
			sentencia = tConexion.createStatement();
			tConexion.setAutoCommit(true);
			resultado = sentencia.executeQuery(sql);
			while (resultado.next()) {
				//System.out.print("   ");
				fila = new Vector<String>();
				for(int i=1;i<=resultado.getMetaData().getColumnCount();i++){
					fila.addElement(resultado.getString(i));
					//System.out.print("|"+resultado.getString(i));
				}
				salida.add(fila);
				fila=null;
				//System.out.println("|");
			}
		} catch (SQLException e) {System.out.println("BBDD.consultaSQL Error sql: "+sql);e.printStackTrace();}
		try {tConexion.close();} catch (SQLException e) {
			System.out.println("BBDD.consultaSQL - No pudo cerrar la base de datos.");
			e.printStackTrace();
		}
		return salida;
	}
	public static Vector<String> consulta1ColumnaSQL(String sql){
		Connection tConexion = null;Statement sentencia = null;ResultSet resultado = null;
		Vector<String> salida = new Vector<String>();
		try {tConexion = DriverManager.getConnection(BBDD.cadenaConexion);
		} catch (SQLException e) {
			System.out.println("BBDD.consulta1ColumnaSQL - No pudo conectar con la base de datos.");
			e.printStackTrace();
			}
		//System.out.println("ConsultaSQL = "+sql);
		try {
			sentencia = tConexion.createStatement();
			tConexion.setAutoCommit(true);
			resultado = sentencia.executeQuery(sql);
			while (resultado.next()) {
				salida.addElement(resultado.getString(1));
				//System.out.println("|"+resultado.getString(1));
			}
		} catch (SQLException e) {System.out.println("BBDD.consulta1ColumnaSQL Error sql: "+sql);e.printStackTrace();}
		try {tConexion.close();} catch (SQLException e) {
			System.out.println("BBDD.consulta1ColumnaSQL No pudo cerrar la base de datos.");
			e.printStackTrace();
		}
		return salida;
	}
	public static Connection abrirConexion(){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("BBDD abrirConexion.");
		Connection conexion = null;
		try {conexion = DriverManager.getConnection(BBDD.cadenaConexion);
		} catch (SQLException e) {e.printStackTrace();}
		return conexion;
	}
	public static void cerrarConexion(Connection conexion){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("BBDD cerrarConexion.");
		try {
			conexion.setAutoCommit(true);
			Statement sentencia = conexion.createStatement();
			sentencia.execute("SHUTDOWN;");
		} catch (SQLException e) {System.out.println("BBDD.cerrarConexion Error al parar la bbbdd.");e.printStackTrace();}
		try {conexion.close();} catch (SQLException e) {
			System.out.println("BBDD.cerrarConexion - No pudo cerrar la base de datos.");
			e.printStackTrace();
		}
	}
	public static String insertarIdSQLAbierta(Connection tConexion,String sql,String tabla){
		String idUltimo = null;
		Statement sentencia = null;
		ResultSet resultado = null;
		try {
			tConexion.setAutoCommit(false);
			sentencia = tConexion.createStatement();
			sentencia.execute("SET AUTOCOMMIT FALSE;");
			sentencia.execute("LOCK TABLE "+tabla+" WRITE;");
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("sql = "+sql);
			sentencia.execute(sql);
			resultado = sentencia.executeQuery("CALL IDENTITY();");
			if (resultado.next()) {
				idUltimo=resultado.getString(1);
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" Ultimo id = "+idUltimo);
			}
			sentencia.execute("COMMIT;");
			tConexion.commit();
		} catch (SQLException e) {
			try {sentencia.execute("ROLLBACK;");tConexion.setAutoCommit(true);} catch (SQLException e1) {e1.printStackTrace();}
			System.out.println("BBDD.insertarIdSQLAbierta Error SQL:"+sql);e.printStackTrace();}
		return idUltimo;
	}
	public static String insertarIdSQL(String tabla,String sql){
		String idUltimo = null;
		Connection tConexion = null;
		try {tConexion = DriverManager.getConnection(BBDD.cadenaConexion);
		} catch (SQLException e) {e.printStackTrace();}
		Statement sentencia = null;
		ResultSet resultado = null;
		try {
			tConexion.setAutoCommit(false);
			sentencia = tConexion.createStatement();
			sentencia.execute("SET AUTOCOMMIT FALSE;");
			sentencia.execute("LOCK TABLE "+tabla+" WRITE;");
			if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar(" sql = "+sql);
			sentencia.execute(sql);
			resultado = sentencia.executeQuery("CALL IDENTITY();");
			if (resultado.next()) {
				idUltimo=resultado.getString(1);
				//System.out.println("insertarIdSQL - Ultimo id = "+idUltimo);
			}
			sentencia.execute("COMMIT;");
			tConexion.commit();
			tConexion.setAutoCommit(true);
			sentencia.execute("SHUTDOWN;");
		} catch (SQLException e) {
			try {sentencia.execute("ROLLBACK;");tConexion.setAutoCommit(true);} catch (SQLException e1) {e1.printStackTrace();}
			System.out.println("BBDD.insertarIdSQL Error SQL:"+sql);e.printStackTrace();}
		try {tConexion.close();} catch (SQLException e) {
			System.out.println("BBDD.insertarIdSQL - No pudo cerrar la base de datos.");
			e.printStackTrace();
		}
		return idUltimo;
	}
	public static int ejecutaSQL(String sql){
		int cambios=0;
		if (sql!=null) if (!sql.equals("")) {
			Connection tConexion = null;
			try {tConexion = DriverManager.getConnection(BBDD.cadenaConexion);
			} catch (SQLException e) {e.printStackTrace();}
			Statement sentencia = null;
			try {
				sentencia = tConexion.createStatement();
				sentencia.execute("SET AUTOCOMMIT FALSE;"); //como una transacción si hay varias sentencias
				
				String[] sentencias=sql.split(";");
				cambios=sentencias.length;
				for (int i=0;i<cambios;i++) {
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("sql ("+i+"/"+sentencias.length+"):["+sentencias[i]+";] => ");
					if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("sql ("+i+"/"+sentencias.length+"): "+sentencia.executeUpdate(sentencias[i]+";")+" registros");
					else sentencia.executeUpdate(sentencias[i]+";");
				}
				//if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("sql: "+sql);
				//cambios=sentencia.executeUpdate(sql);
				
				tConexion.commit();
				sentencia.execute("COMMIT;");
				sentencia.execute("SHUTDOWN;");
			} catch (SQLException e) {e.printStackTrace();}
			try {tConexion.close();} catch (SQLException e) {
				System.out.println("BBDD.ejecutaSQL - No pudo cerrar la base de datos.");
				e.printStackTrace();
			}
		} else System.out.println("BBDD.ejecutaSQL - Sentencia sql vacia.");
		return cambios;
	}
	public static int ejecutaVariasSQL(Vector<String> sql){
		int cambios=0;
		Connection tConexion = null;
		String sqlSentencia="";
		try {tConexion = DriverManager.getConnection(BBDD.cadenaConexion);
		} catch (SQLException e) {e.printStackTrace();}
		Statement sentencia = null;
		try {
			sentencia = tConexion.createStatement();
			for (;cambios<sql.size();cambios++) {
				//System.out.println("ejecutaVariasSQL sql("+i+"):"+sql.get(i));
				sqlSentencia=sql.get(cambios);
				sentencia.execute(sqlSentencia);
				//sentencia.execute("COMMIT;");
			}
			tConexion.commit();
			sentencia.execute("COMMIT;");
			sentencia.execute("SHUTDOWN;");
		} catch (SQLException e) {System.out.println("BBDD.ejecutaVariasSQL Error sql:"+sqlSentencia);e.printStackTrace();}
		try {tConexion.close();} catch (SQLException e) {
			System.out.println("BBDD.ejecutaVariasSQL No pudo cerrar la base de datos.");
			e.printStackTrace();
		}
		return cambios;
	}
	public static void crear(){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Creando tablas...");
		Vector<String> sentencias = new Vector<String>();
		
		// SUSTITUIR POR bbdd.java
		sentencias.addElement("CREATE TABLE PROFESORES(ID INT NOT NULL IDENTITY, NOMBRE VARCHAR(256) NOT NULL, APELLIDOS VARCHAR(256) NOT NULL, DATOS VARCHAR(256));");
		sentencias.addElement("CREATE TABLE USUARIO(PROFESOR INT NOT NULL);");
		sentencias.addElement("CREATE TABLE ENSENANZAS(ID INT NOT NULL IDENTITY, REFERENCIA VARCHAR(5) NOT NULL, DESCRIPCION VARCHAR(1024) NOT NULL);");
		sentencias.addElement("CREATE TABLE TITULOS(ID INT NOT NULL IDENTITY, ENSENANZA INT NOT NULL, REFERENCIA VARCHAR(5) NOT NULL, DESCRIPCION VARCHAR(1024) NOT NULL);");
		sentencias.addElement("CREATE TABLE NIVELES(ID INT NOT NULL IDENTITY, TITULO INT NOT NULL, REFERENCIA VARCHAR(3) NOT NULL);");
		sentencias.addElement("CREATE TABLE MATERIAS(ID INT NOT NULL IDENTITY, REFERENCIA VARCHAR(5) NOT NULL, DESCRIPCION VARCHAR(256) NOT NULL, NIVEL INT);");
		sentencias.addElement("CREATE TABLE CLASES(ID INT NOT NULL IDENTITY, NIVEL INT, REFERENCIA VARCHAR(5) NOT NULL, TUTOR INT NOT NULL);");
		sentencias.addElement("CREATE TABLE MATERIA_CLASE(ID INT NOT NULL IDENTITY, MATERIA INT NOT NULL, CLASE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE UH_SEMANA(ID INT NOT NULL IDENTITY, MATERIACLASE INT NOT NULL, FIN TIME NOT NULL, INICIO TIME NOT NULL, DIA_SEMANA INT NOT NULL);");
		sentencias.addElement("CREATE TABLE ESTUDIANTES(ID INT NOT NULL IDENTITY, NOMBRE VARCHAR(256) NOT NULL, APELLIDOS VARCHAR(256) NOT NULL, DATOS VARCHAR(1024), CLASE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE ESTUDIANTE_MATERIACLASE(ESTUDIANTE INT NOT NULL, MATERIACLASE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE TIPOS_EVALUABLE(ID INT NOT NULL IDENTITY, REFERENCIA VARCHAR(3) NOT NULL, DESCRIPCION VARCHAR(256), PESO INT NOT NULL, MATERIA INT, MINIMO DECIMAL(3,1) DEFAULT 0 NOT NULL);");
		sentencias.addElement("CREATE TABLE EVALUABLES(ID INT NOT NULL IDENTITY, DIA_INICIO DATE NOT NULL, UH_INICIO INT NOT NULL, DIA_FIN DATE NOT NULL, UH_FIN INT NOT NULL, REFERENCIA VARCHAR(8) NOT NULL, ENUNCIADO VARCHAR(512) NOT NULL, PESO INT NOT NULL, TIPO INT NOT NULL);");
		sentencias.addElement("CREATE TABLE EVALUACIONES_EVALUABLES(ID INT NOT NULL IDENTITY, EVALUABLE INT NOT NULL, NOTA DECIMAL(3,1) DEFAULT 0 NOT NULL, OBSERVACIONES VARCHAR(512), GRUPO VARCHAR(10));");
		sentencias.addElement("CREATE TABLE EE_ESTUDIANTES(E_EVALUABLE INT NOT NULL, ESTUDIANTE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE TIPOS_EVENTO(ID INT NOT NULL, DESCRIPCION VARCHAR(64) NOT NULL, COLOR INT NOT NULL, BBDD VARCHAR(255));");
		sentencias.addElement("CREATE TABLE EVENTOS(ID INT NOT NULL IDENTITY, DIA DATE NOT NULL, HORA TIME NOT NULL, SUMARIO VARCHAR(1024), PREPARATIVOS VARCHAR(512), CONCLUSION VARCHAR(512), TIPO INT NOT NULL);");
		sentencias.addElement("CREATE TABLE CITAS_TUTORES(ID INT NOT NULL, RELACION VARCHAR(256));");
		sentencias.addElement("CREATE TABLE ESTUDIANTES_CITATUTORES(ESTUDIANTE INT NOT NULL, EVENTO_CITATUTORES INT NOT NULL);");
		sentencias.addElement("CREATE TABLE EVALUACIONES(ID INT NOT NULL, DESCRIPCION VARCHAR(256) NOT NULL, CLASE INT NOT NULL);");
		sentencias.addElement("CREATE TABLE CLAUSTROS(ID INT NOT NULL, ORDINARIO BOOLEAN);");
		sentencias.addElement("CREATE TABLE ETCPS(ID INT NOT NULL);");
		sentencias.addElement("CREATE TABLE C_ESCOLARES(ID INT NOT NULL, ORDINARIO BOOLEAN);");
		sentencias.addElement("CREATE TABLE OTROS_EVENTOS(ID INT NOT NULL, DESCRIPCION VARCHAR(256));");
		sentencias.addElement("CREATE TABLE DIASNOLECTIVOS(ID INT NOT NULL IDENTITY, CLASE INT, FECHA DATE NOT NULL, DESCRIPCION VARCHAR(256));");
		sentencias.addElement("CREATE TABLE PROGRAMACION_NODO(ID INT NOT NULL IDENTITY, PADRE INT, POSICION INT NOT NULL, MATERIA INT, ENUNCIADO VARCHAR(256) NOT NULL, SUMARIO VARCHAR(1024));");
		sentencias.addElement("CREATE TABLE PROGRAMACION_TEMPORAL(NODO INT NOT NULL, CLASE INT NOT NULL, DURACION INTEGER NOT NULL);");
		sentencias.addElement("CREATE TABLE PARCIALES(ID INT NOT NULL IDENTITY, MATERIA INT NOT NULL, INICIO DATE NOT NULL, FIN DATE NOT NULL);");
		sentencias.addElement("ALTER TABLE USUARIO ADD CONSTRAINT IDX_USUARIO_PK PRIMARY KEY (PROFESOR);");
		sentencias.addElement("ALTER TABLE ESTUDIANTE_MATERIACLASE ADD CONSTRAINT IDX_ESTUDIANTE_MATERIACLASE_PK PRIMARY KEY (ESTUDIANTE, MATERIACLASE);");
		sentencias.addElement("ALTER TABLE EE_ESTUDIANTES ADD CONSTRAINT IDX_EE_ESTUDIANTES_PK PRIMARY KEY (E_EVALUABLE, ESTUDIANTE);");
		sentencias.addElement("ALTER TABLE TIPOS_EVENTO ADD CONSTRAINT IDX_TIPOS_EVENTO_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE CITAS_TUTORES ADD CONSTRAINT IDX_CITAS_TUTORES_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTES_CITATUTORES ADD CONSTRAINT IDX_ESTUDIANTES_CITATUTORES_PK PRIMARY KEY (ESTUDIANTE, EVENTO_CITATUTORES);");
		sentencias.addElement("ALTER TABLE CLAUSTROS ADD CONSTRAINT IDX_CLAUSTROS_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE ETCPS ADD CONSTRAINT IDX_ETCPS_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE C_ESCOLARES ADD CONSTRAINT IDX_C_ESCOLARES_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE OTROS_EVENTOS ADD CONSTRAINT IDX_OTROS_EVENTOS_PK PRIMARY KEY (ID);");
		sentencias.addElement("ALTER TABLE PROGRAMACION_TEMPORAL ADD CONSTRAINT IDX_PROGRAMACION_TEMPORAL_PK PRIMARY KEY (NODO, CLASE);");
		sentencias.addElement("ALTER TABLE USUARIO ADD CONSTRAINT IDX_USUARIO_FK0 FOREIGN KEY (PROFESOR) REFERENCES PROFESORES (ID);");
		sentencias.addElement("ALTER TABLE TITULOS ADD CONSTRAINT IDX_TITULOS_FK0 FOREIGN KEY (ENSENANZA) REFERENCES ENSENANZAS (ID);");
		sentencias.addElement("ALTER TABLE NIVELES ADD CONSTRAINT IDX_NIVELES_FK0 FOREIGN KEY (TITULO) REFERENCES TITULOS (ID);");
		sentencias.addElement("ALTER TABLE MATERIAS ADD CONSTRAINT IDX_MATERIAS_FK0 FOREIGN KEY (NIVEL) REFERENCES NIVELES (ID);");
		sentencias.addElement("ALTER TABLE CLASES ADD CONSTRAINT IDX_CLASES_FK0 FOREIGN KEY (TUTOR) REFERENCES PROFESORES (ID);");
		sentencias.addElement("ALTER TABLE CLASES ADD CONSTRAINT IDX_CLASES_FK1 FOREIGN KEY (NIVEL) REFERENCES NIVELES (ID);");
		sentencias.addElement("ALTER TABLE MATERIA_CLASE ADD CONSTRAINT IDX_MATERIA_CLASE_FK0 FOREIGN KEY (MATERIA) REFERENCES MATERIAS (ID);");
		sentencias.addElement("ALTER TABLE MATERIA_CLASE ADD CONSTRAINT IDX_MATERIA_CLASE_FK1 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE UH_SEMANA ADD CONSTRAINT IDX_UH_SEMANA_FK0 FOREIGN KEY (MATERIACLASE) REFERENCES MATERIA_CLASE (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTES ADD CONSTRAINT IDX_ESTUDIANTES_FK0 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTE_MATERIACLASE ADD CONSTRAINT IDX_ESTUDIANTE_MATERIACLASE_FK0 FOREIGN KEY (ESTUDIANTE) REFERENCES ESTUDIANTES (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTE_MATERIACLASE ADD CONSTRAINT IDX_ESTUDIANTE_MATERIACLASE_FK1 FOREIGN KEY (MATERIACLASE) REFERENCES MATERIA_CLASE (ID);");
		sentencias.addElement("ALTER TABLE TIPOS_EVALUABLE ADD CONSTRAINT IDX_TIPOS_EVALUABLE_FK0 FOREIGN KEY (MATERIA) REFERENCES MATERIAS (ID);");
		sentencias.addElement("ALTER TABLE EVALUABLES ADD CONSTRAINT IDX_EVALUABLES_FK0 FOREIGN KEY (UH_INICIO) REFERENCES UH_SEMANA (ID);");
		sentencias.addElement("ALTER TABLE EVALUABLES ADD CONSTRAINT IDX_EVALUABLES_FK1 FOREIGN KEY (TIPO) REFERENCES TIPOS_EVALUABLE (ID);");
		sentencias.addElement("ALTER TABLE EVALUABLES ADD CONSTRAINT IDX_EVALUABLES_FK2 FOREIGN KEY (UH_FIN) REFERENCES UH_SEMANA (ID);");
		sentencias.addElement("ALTER TABLE EVALUACIONES_EVALUABLES ADD CONSTRAINT IDX_EVALUACIONES_EVALUABLES_FK0 FOREIGN KEY (EVALUABLE) REFERENCES EVALUABLES (ID);");
		sentencias.addElement("ALTER TABLE EE_ESTUDIANTES ADD CONSTRAINT IDX_EE_ESTUDIANTES_FK0 FOREIGN KEY (E_EVALUABLE) REFERENCES EVALUACIONES_EVALUABLES (ID);");
		sentencias.addElement("ALTER TABLE EE_ESTUDIANTES ADD CONSTRAINT IDX_EE_ESTUDIANTES_FK1 FOREIGN KEY (ESTUDIANTE) REFERENCES ESTUDIANTES (ID);");
		sentencias.addElement("ALTER TABLE EVENTOS ADD CONSTRAINT IDX_EVENTOS_FK0 FOREIGN KEY (TIPO) REFERENCES TIPOS_EVENTO (ID);");
		sentencias.addElement("ALTER TABLE CITAS_TUTORES ADD CONSTRAINT IDX_CITAS_TUTORES_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTES_CITATUTORES ADD CONSTRAINT IDX_ESTUDIANTES_CITATUTORES_FK0 FOREIGN KEY (ESTUDIANTE) REFERENCES ESTUDIANTES (ID);");
		sentencias.addElement("ALTER TABLE ESTUDIANTES_CITATUTORES ADD CONSTRAINT IDX_ESTUDIANTES_CITATUTORES_FK1 FOREIGN KEY (EVENTO_CITATUTORES) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE EVALUACIONES ADD CONSTRAINT IDX_EVALUACIONES_FK0 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE EVALUACIONES ADD CONSTRAINT IDX_EVALUACIONES_FK1 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE CLAUSTROS ADD CONSTRAINT IDX_CLAUSTROS_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE ETCPS ADD CONSTRAINT IDX_ETCPS_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE C_ESCOLARES ADD CONSTRAINT IDX_C_ESCOLARES_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE OTROS_EVENTOS ADD CONSTRAINT IDX_OTROS_EVENTOS_FK0 FOREIGN KEY (ID) REFERENCES EVENTOS (ID);");
		sentencias.addElement("ALTER TABLE DIASNOLECTIVOS ADD CONSTRAINT IDX_DIASNOLECTIVOS_FK0 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE PROGRAMACION_NODO ADD CONSTRAINT IDX_PROGRAMACION_NODO_FK0 FOREIGN KEY (MATERIA) REFERENCES MATERIAS (ID);");
		sentencias.addElement("ALTER TABLE PROGRAMACION_TEMPORAL ADD CONSTRAINT IDX_PROGRAMACION_TEMPORAL_FK0 FOREIGN KEY (NODO) REFERENCES PROGRAMACION_NODO (ID);");
		sentencias.addElement("ALTER TABLE PROGRAMACION_TEMPORAL ADD CONSTRAINT IDX_PROGRAMACION_TEMPORAL_FK1 FOREIGN KEY (CLASE) REFERENCES CLASES (ID);");
		sentencias.addElement("ALTER TABLE PARCIALES ADD CONSTRAINT IDX_PARCIALES_FK0 FOREIGN KEY (MATERIA) REFERENCES MATERIAS (ID);");
		//HASTA AQUÍ
		
		System.out.println("Creando base de datos ... "+sentencias.size()+" sentencias. ");
		ejecutaVariasSQL(sentencias);
		
		sentencias.clear();
		System.out.print("Insertando valores de variables... ");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoCitaConTutores+",'CITAS_TUTORES','Cita con Tutores', "+SWT.COLOR_RED+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoEvaluacion+",'EVALUACIONES','Evaluación', "+SWT.COLOR_GREEN+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoClaustro+",'CLAUSTROS','Claustro', "+SWT.COLOR_MAGENTA+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoETCP+",'ETCPS','ETCP', "+SWT.COLOR_DARK_GREEN+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoConsejoEscolar+",'C_ESCOLARES','Consejo Escolar', "+SWT.COLOR_DARK_CYAN+");");
		sentencias.addElement("INSERT INTO TIPOS_EVENTO(ID,BBDD,DESCRIPCION,COLOR) VALUES ("+BBDD.idTipoOtroTipo+",'OTROS_EVENTOS','Otro tipo', "+SWT.COLOR_CYAN+");");

		System.out.println(ejecutaVariasSQL(sentencias)+"/"+sentencias.size());
	}
	public static void valoresIniciales(){

		Vector<String> sentencias = new Vector<String>();
		System.out.print("Insertando valores iniciales... ");
		// Solo crear el usuario ...
		sentencias.addElement("INSERT INTO PROFESORES (NOMBRE,APELLIDOS) VALUES ('Profesor','...');");
		sentencias.addElement("INSERT INTO USUARIO (PROFESOR) VALUES (0);");

		// Crear valores de prueba ...
	
		System.out.println(ejecutaVariasSQL(sentencias)+"/"+sentencias.size());
	}
	public static void borrarTablas(){
		System.out.println("Borrando tablas...");
		Vector<String> sentencias = new Vector<String>();
		sentencias.addElement("DROP TABLE ESTUDIANTES_CITA IF EXISTS;");
		sentencias.addElement("DROP TABLE EVENTOS IF EXISTS;");
		sentencias.addElement("DROP TABLE OTROS_EVENTOS IF EXISTS;");
		sentencias.addElement("DROP TABLE C_ESCOLARES IF EXISTS;");
		sentencias.addElement("DROP TABLE ETCPS IF EXISTS;");
		sentencias.addElement("DROP TABLE CLAUSTROS IF EXISTS;");
		sentencias.addElement("DROP TABLE EVALUACIONES IF EXISTS;");
		sentencias.addElement("DROP TABLE TIPOS_EVENTO IF EXISTS;");
		sentencias.addElement("DROP TABLE CITAS_TUTORES IF EXISTS;");
		sentencias.addElement("DROP TABLE OBSERVACION_ESTUDIANTES IF EXISTS;");
		sentencias.addElement("DROP TABLE OBSERVACIONES IF EXISTS;");
		sentencias.addElement("DROP TABLE IS_ESTUDIANTES IF EXISTS;");
		sentencias.addElement("DROP TABLE SUCESOS IF EXISTS;");
		sentencias.addElement("DROP TABLE EE_ESTUDIANTES IF EXISTS;");
		sentencias.addElement("DROP TABLE EVALUACIONES_EVALUABLES IF EXISTS;");
		sentencias.addElement("DROP TABLE EVALUABLES IF EXISTS;");
		sentencias.addElement("DROP TABLE FALTAS IF EXISTS;");
		sentencias.addElement("DROP TABLE ESTUDIANTE_MATERIACLASE IF EXISTS;");
		sentencias.addElement("DROP TABLE ESTUDIANTES IF EXISTS;");
		sentencias.addElement("DROP TABLE CURSO IF EXISTS;");
		sentencias.addElement("DROP TABLE MATERIA_CLASE IF EXISTS;");
		sentencias.addElement("DROP TABLE CLASES IF EXISTS;");
		sentencias.addElement("DROP TABLE PROFESORES IF EXISTS;");
		sentencias.addElement("DROP TABLE UH_SEMANA IF EXISTS;");
		sentencias.addElement("DROP TABLE MATERIAS IF EXISTS;");
		sentencias.addElement("DROP TABLE PROFESOR IF EXISTS;");
		for (int i=0;i<sentencias.size();i++) {
			BBDD.ejecutaSQL(sentencias.get(i));
			System.out.print(".");
		}
		System.out.println("");
	}
}
