/** Copyright 2012
*
* Author: Ismael PONCE GORDILLO
*
* Licensed under the EUPL, Version 1.1 or – as soon they will be
* approved by the European Commission – subsequent versions of the
* EUPL (the "Licence"); you may not use this work except in
* compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* https://joinup.ec.europa.eu/software/page/eupl/licence-eupl
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the Licence is distributed on an "AS
* IS" BASIS, * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
* express or implied.
* See the Licence for the specific language governing permissions
* and limitations under the Licence.
*/
import java.io.File;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Monitor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.TabItem;

public class PCuaderno {
	public static PCuaderno MiPCuaderno=null;
	public Display display=null;
	public Shell shellPrincipal = null;
	public String titulo="pCuaderno";
	public String descripcion="Herramienta de apoyo para el profesor/a de enseñanzas presenciales";
	public String usando="Usando librerías graficas SWT\nUsando librerías gestor bases de datos HSQLDB";
	public String version="0.1";
	private TabFolder tabFolderClasificador = null;
	private TabItem tabItemClase; public final String tituloClase="Clase"; 
	private TabItem tabItemHorario; public final String tituloHorario="Horario";
	private TabItem tabItemCalendario; public final String tituloCalendario="Calendario";
	private TabItem tabItemEventos; public final String tituloEventos="Eventos";
	private TabItem tabItemEvaluables; public final String tituloEvaluables="Evaluables";
	private TabItem tabItemProgramacion; public final String tituloProgramacion="Programación";
	private TabItem tabItemMantenimiento; public final String tituloMantenimiento="Mantenimiento";
	private TabItem tabItemMatricula; public final String tituloMatricula="Matrícula";
	public GrupoClase grupoClase = null;
	public Horario horario = null;
	public Calendario calendario = null;
	public Evento evento = null;
	public Evaluable evaluable = null;
	public Programacion programacion = null;
	public Mantenimiento mantenimiento = null;
	public Matricula matricula = null;
	private Integer nivelNotificable=0;
	public static final boolean noti=false;

	public void notificar(String texto,boolean visible){
		String clase=Thread.currentThread().getStackTrace()[2].getClassName();
		String metodo=Thread.currentThread().getStackTrace()[2].getMethodName();
		String tabulador=((metodo.equals("widgetSelected"))?"--":"  ");
		String sangria="";for (int i=Thread.currentThread().getStackTrace().length;i>3;i--) sangria=sangria+tabulador;
		if (visible) nivelNotificable--;
		else nivelNotificable++;
		System.out.println(sangria+nivelNotificable+": "+clase+"."+metodo+": "+texto);
	}
	public void notificar(String texto){
		if (PCuaderno.MiPCuaderno.nivelNotificable.equals(0)) {
			String clase=Thread.currentThread().getStackTrace()[2].getClassName();
			String metodo=Thread.currentThread().getStackTrace()[2].getMethodName();
			//String traza="";
			String tabulador=((metodo.equals("widgetSelected"))?"--":"  ");
			String sangria="";for (int i=Thread.currentThread().getStackTrace().length;i>3;i--) {
				sangria=sangria+tabulador;
				//traza=traza+Thread.currentThread().getStackTrace()[i-1].getClassName()+"."+Thread.currentThread().getStackTrace()[i-1].getMethodName()+">";
			}
			System.out.println(sangria+clase+"."+metodo+": "+texto);//+"  FechaTrabajo="+Fecha.formaFecha(Fecha.fechaDeTrabajo));//+" |||"+traza);
		}
	}
	public void informacion(String m){
		MessageBox messageBox = new MessageBox(PCuaderno.MiPCuaderno.shellPrincipal,SWT.ICON_INFORMATION | SWT.OK);
		messageBox.setText(titulo+" - Información");messageBox.setMessage(m);messageBox.open();
	}
	public int pregunta(String m){
		MessageBox messageBox = new MessageBox(PCuaderno.MiPCuaderno.shellPrincipal,SWT.ICON_QUESTION | SWT.OK | SWT.CANCEL);
		messageBox.setText(titulo+" - Pregunta");messageBox.setMessage(m);return messageBox.open();
	}
	public void error(String m){
		MessageBox messageBox = new MessageBox(PCuaderno.MiPCuaderno.shellPrincipal,SWT.ICON_ERROR | SWT.OK);
		messageBox.setText(titulo+" - Error");messageBox.setMessage(m);messageBox.open();
	}
	public int aviso(String m){
		MessageBox messageBox = new MessageBox(PCuaderno.MiPCuaderno.shellPrincipal,SWT.ICON_WARNING | SWT.OK | SWT.CANCEL);
		messageBox.setText(titulo+" - Aviso");messageBox.setMessage(m);return messageBox.open();
	}
	public String entrada(String campo,String texto,boolean clave){
		Entrada entrada = new Entrada(PCuaderno.MiPCuaderno.shellPrincipal,campo,texto,clave);
		return entrada.open();
	}
	public String md5(String cadena){
		String md5Cadena=null;
		if (cadena!=null) {
			MessageDigest md = null;
			try {md = MessageDigest.getInstance("MD5");
			} catch (NoSuchAlgorithmException e) {e.printStackTrace();}
			md.reset();
	        byte[] md5Digest = md.digest(cadena.getBytes());
	        BigInteger md5Numero = new BigInteger(1, md5Digest);
	        md5Cadena = md5Numero.toString(16);
	        md5Cadena = (md5Cadena.length()==31)?"0"+md5Cadena:md5Cadena;
		}
		return md5Cadena;
	}
	private void createTabFolderPrincipal() {
		GridData gridDataClasificador = new GridData();
		gridDataClasificador.horizontalAlignment = GridData.FILL;
		gridDataClasificador.grabExcessHorizontalSpace = true;
		gridDataClasificador.grabExcessVerticalSpace = true;
		gridDataClasificador.verticalAlignment = GridData.FILL;
		tabFolderClasificador = new TabFolder(shellPrincipal, SWT.BORDER);
		tabFolderClasificador.setLayoutData(gridDataClasificador);
		tabItemClase = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemClase.setText(tituloClase);
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Creando clase...");//Parcial.imprimirParciales();
		createClase();
		tabItemClase.setControl(grupoClase);
		tabItemHorario = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemHorario.setText(tituloHorario);
		tabItemHorario.setToolTipText("Ver horario semanal.");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Creando horario...");//Parcial.imprimirParciales();
		createHorario();
		tabItemHorario.setControl(horario);
		tabItemCalendario = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemCalendario.setText(tituloCalendario);
		tabItemCalendario.setToolTipText("Ver horario semanal.");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Creando calendario...");//Parcial.imprimirParciales();
		createCalendario();
		tabItemCalendario.setControl(calendario);
		tabItemEventos = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemEventos.setText(tituloEventos);
		tabItemEventos.setToolTipText("Ver eventos.");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Creando eventos...");//Parcial.imprimirParciales();
		createEvento();
		tabItemEventos.setControl(evento);
		tabItemEvaluables = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemEvaluables.setText(tituloEvaluables);
		tabItemEvaluables.setToolTipText("Ver elementos evaluables.");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Creando evaluables...");//Parcial.imprimirParciales();
		createEvaluable();
		tabItemEvaluables.setControl(evaluable);
		tabItemProgramacion = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemProgramacion.setText(tituloProgramacion);
		tabItemProgramacion.setToolTipText("Gestión de la programación.");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Creando gestion de la programación...");//Parcial.imprimirParciales();
		createProgramacion();
		tabItemProgramacion.setControl(programacion);
		tabItemMantenimiento = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemMantenimiento.setText(tituloMantenimiento);
		tabItemMantenimiento.setToolTipText("Dias no laborables, parciales, tipos de evaluables y profesores.");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Creando mantener...");//Parcial.imprimirParciales();
		createMantener();
		tabItemMantenimiento.setControl(mantenimiento);
		tabItemMatricula = new TabItem(tabFolderClasificador, SWT.NONE);
		tabItemMatricula.setText(tituloMatricula);
		tabItemMatricula.setToolTipText("Estudiantes, clases, matrículas...");
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Creando matriculas...");//Parcial.imprimirParciales();
		createMatricula();
		tabItemMatricula.setControl(matricula);
		tabFolderClasificador
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
					public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
						if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("tabFolderClasificador");
						//if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("widgetSelected()"+((TabItem) e.item).getText());
						if (((TabItem) e.item).getText().equals(tituloClase)){
							grupoClase.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloHorario)){
							horario.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloCalendario)){
							calendario.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloEventos)){
							evento.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloEvaluables)){
							evaluable.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloProgramacion)){
							programacion.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloMantenimiento)){
							mantenimiento.actualizar();
						} else if (((TabItem) e.item).getText().equals(tituloMatricula)){
							matricula.actualizar();
						}
					}
				});
	}
	public void createClase(){grupoClase = new GrupoClase(tabFolderClasificador,SWT.NONE);}
	public void createHorario(){horario = new Horario(tabFolderClasificador,SWT.NONE);}
	public void createCalendario(){calendario = new Calendario(tabFolderClasificador,SWT.NONE);}
	public void createEvento(){evento = new Evento(tabFolderClasificador,SWT.NONE);}
	public void createEvaluable(){evaluable = new Evaluable(tabFolderClasificador,SWT.NONE);}
	public void createProgramacion(){programacion = new Programacion(tabFolderClasificador,SWT.NONE);}
	public void createMantener(){mantenimiento = new Mantenimiento(tabFolderClasificador,SWT.NONE);}
	public void createMatricula(){matricula = new Matricula(tabFolderClasificador,SWT.NONE);}
	public void iniciar(){
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Iniciando Fecha...");
		Fecha.inicializar();
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Iniciando MateriaClase...");
		MateriaClase.inicializar();
		if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Iniciando Parcial...");
		Parcial.inicializar();
	}

	public static void main(String[] args) {
		System.out.println("Iniciando programa...");
		System.out.println("Creando clase PCuaderno...");
		PCuaderno thisClass = new PCuaderno();
		PCuaderno.MiPCuaderno=thisClass;
		thisClass.display = Display.getDefault();
		thisClass.createSShellPricipal();
		boolean vacio=!(new File(BBDD.archivoSCRIPT)).exists();
		System.out.println("Preguntando contraseña...");
		String m=((vacio)?"El programa esta sin información. Esto puede ser por que es la " +
				"primera vez que entra o por que se han borrado los archivos que contienen " +
				"la informaicón.\n\nLa clave no se puede recuperar de ninguna forma, si la " +
				"olvida perderá toda la información. Introduzca la clave que utilizará a partir de ahora " +
				"para acceder al programa.":"Introduzca la clave para acceder al programa.\n\n" +
				"Si olvidó la clave, no podrá recuperar los datos. Solo podrá quitar los archivos " +
				"de la carpeta '"+PCuaderno.MiPCuaderno.titulo+"_bbdd"+"' de donde este ubicado el programa y así le dejará empezar " +
				"de nuevo con una clave nueva.")+"\n\nLicencia EUPLv1.1 Copyright 2012";
		String acceso=PCuaderno.MiPCuaderno.md5(PCuaderno.MiPCuaderno.entrada("Clave",m,true));
		if (acceso!=null) {
			if (BBDD.acceso(acceso)){
				acceso=null;
				System.out.println("Quitando bloqueo de la base de datos si existe.");
				BBDD.borrarArchivo(BBDD.archivoLCK);
				if (vacio) {BBDD.crear();BBDD.valoresIniciales();}
				thisClass.iniciar();
				thisClass.createTabFolderPrincipal();
				Monitor primary = thisClass.display.getPrimaryMonitor ();
				Rectangle bounds = primary.getBounds ();
				Rectangle rect = thisClass.shellPrincipal.getBounds ();
				int x = bounds.x + (bounds.width - rect.width) / 2;
				int y = bounds.y + (bounds.height - rect.height) / 2;
				thisClass.shellPrincipal.setLocation (x, y);
				thisClass.shellPrincipal.open();
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("Listo -----------------------");
				if (PCuaderno.noti) PCuaderno.MiPCuaderno.notificar("-----------------------------");
				while (!thisClass.shellPrincipal.isDisposed()) {
					if (!thisClass.display.readAndDispatch()) {
						thisClass.display.sleep();
						thisClass.display.readAndDispatch();
					}
				}
				TextoDemorado.escribeSiHay();
				BBDD.cerrarBBDD();
			} else {
				m="No pudo conectar con la base de datos, probablemente sea por que la clave" +
						" introducida no es correcta. El programa se cerrará.";
				PCuaderno.MiPCuaderno.informacion(m);
			}
			acceso=null;
		}
		thisClass.shellPrincipal.dispose();
		thisClass.display.dispose();
	}
	private void createSShellPricipal() {
		GridLayout gridLayout = new GridLayout();
		gridLayout.horizontalSpacing = 2;
		gridLayout.marginWidth = 2;
		gridLayout.marginHeight = 2;
		gridLayout.verticalSpacing = 2;
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.verticalAlignment = GridData.CENTER;
		shellPrincipal = new Shell(display);
		shellPrincipal.setText(titulo);
		shellPrincipal.setLayout(gridLayout);
		shellPrincipal.setSize(new Point(700, 400));
	}
	public void seleccionarFicha(String titulo){
		if (titulo.equals(tituloClase))
			tabFolderClasificador.setSelection(tabItemClase);
		else if (titulo.equals(tituloHorario))
			tabFolderClasificador.setSelection(tabItemHorario);
		else if (titulo.equals(tituloCalendario))
			tabFolderClasificador.setSelection(tabItemCalendario);
		else if (titulo.equals(tituloEventos))
			tabFolderClasificador.setSelection(tabItemEventos);
		else if (titulo.equals(tituloEvaluables))
			tabFolderClasificador.setSelection(tabItemEvaluables);
		else if (titulo.equals(tituloProgramacion))
			tabFolderClasificador.setSelection(tabItemProgramacion);
		else if (titulo.equals(tituloMantenimiento))
			tabFolderClasificador.setSelection(tabItemMantenimiento);
		else if (titulo.equals(tituloMatricula))
			tabFolderClasificador.setSelection(tabItemMatricula);
	}
}
